unit EccAdmin;

interface

uses
  Generics.Collections, EccMemos, XmlIntf;

type
  TAdminNode = class
    Id, NodeType, CongrType: Integer;
    FPerson: Integer;
    Title, FPlace, NastName, ServiceId: string;
    // FPlace   Glory  NodeType = 3
    Childs: TList<TAdminNode>;
    Parent: TAdminNode;
    constructor Create;
    destructor Destroy; override;
    function GetPerson: TPerson;
    function GetPath: string;
    function GetPlace(Index: Integer): string;
    procedure SetPlace(Index: Integer; Value: string);
    property Person: TPerson read GetPerson;
    property Path: string read GetPath;
    property Place: string index 0 read GetPlace write SetPlace;
    property GloryId: string index 1 read GetPlace write SetPlace;
    property Holi: string index 2 read GetPlace write SetPlace;
    property SubMem: string index 3 read GetPlace write SetPlace;
  end;

var
  AdminDiv: TAdminNode;

const
  _AdminLevels: array[0..3] of string = ('pomest', 'div', 'congr', 'chap');

procedure LoadPomest(const Node: IXmlNode);
function CreateNewAdminSubnode(AN: TAdminNode): TAdminNode;

implementation

uses
  SysUtils, StrUtils, EccProcs, EccXml;

{ TAdminNode }

constructor TAdminNode.Create;
begin
  Childs := TList<TAdminNode>.Create;
end;

destructor TAdminNode.Destroy;
var
  i: TAdminNode;
begin
  if Childs <> nil then
    begin
      for i in Childs do i.Destroy;
      Childs.Free;
    end;
  inherited;
end;

function TAdminNode.GetPath: string;
var
  o: TAdminNode;
begin
  o := Self;
  Result := '';
  repeat
    Result := IntToStr(o.Id) + '.' + Result;
    o := o.Parent;
  until o = AdminDiv;
  SetLength(Result, Length(Result) - 1);
end;

function TAdminNode.GetPerson: TPerson;
var
  o: TAdminNode;
begin
  o := Self;
  repeat
    try
      Result := PersonsDic[o.FPerson];
    except
      Result := nil;
    end;
    o := o.Parent;
  until (Result <> nil) or (o.Parent = nil);
end;

procedure DispatchMemory(const S: string; out S1, S2, S3: string);
begin {opt}
  StrDivide(S, S1, S2, '.');
  StrDivide(S2, S2, S3, '.');
end;

function TAdminNode.GetPlace(Index: Integer): string;
var
  s1, s2, s3: string;
begin
  if NodeType > 1 then
    if Index = 0 then Result := ''
    else
      begin
        DispatchMemory(FPlace, s1, s2, s3);
        case Index of
          1: Result := s1;
          2: Result := s2;
          3: Result := s3;
        end;
      end
  else
    case Index of
      0: Result := FPlace;
      else Result := '';
    end;
end;

procedure TAdminNode.SetPlace(Index: Integer; Value: string);
var
  s1, s2, s3: string;
begin
  if NodeType > 1 then
    if Index > 0 then
      begin
        DispatchMemory(FPlace, s1, s2, s3);
        case Index of
          1: FPlace := Value + '.' + s2 + '.' + s3;
          2: FPlace := s1 + '.' + Value + '.' + s3;
          3: FPlace := s1 + '.' + s2 + '.' + Value;
        end;
      end
    else
  else
    if Index = 0 then FPlace := Value;
end;

procedure LoadPomest(const Node: IXmlNode);
  procedure _recurse(Node: TAdminNode; XmlNode: IXmlNode);
  var
    i: Integer;
    ti: TAdminNode;
  begin
    ti := TAdminNode.Create;
    ti.Parent := Node;
    Node.Childs.Add(ti);
    ti.Title := GetAttr(XmlNode, 'disp');
    ti.NodeType := IndexStr(XmlNode.NodeName, _AdminLevels);
    ti.Id := GetNumAttr(XmlNode, 'id');
    case ti.NodeType of
      0..1: begin
        ti.FPerson := GetNumAttr(XmlNode, 'pers');
        ti.FPlace := GetAttr(XmlNode, 'place');
      end;
      2: begin
        ti.CongrType := GetNumAttr(XmlNode, 'type');
        ti.NastName := GetAttr(XmlNode, 'nast');
      end;
      3: begin
        ti.FPlace := GetAttr(XmlNode, 'glory');
        ti.ServiceId := GetAttr(XmlNode, 'serv');
      end;
    end;
    for i := 0 to XmlNode.ChildNodes.Count - 1 do
      _recurse(ti, XmlNode.ChildNodes[i]);
  end;
begin
  _recurse(AdminDiv, Node);
end;

function CreateNewAdminSubnode(AN: TAdminNode): TAdminNode;
var
  i, j: Integer;
begin
  j := 1;
  for i := 0 to AN.Childs.Count - 1 do
    if AN.Childs[i].Id >= j then j := AN.Childs[i].Id + 1;
  Result := TAdminNode.Create;
  Result.Parent := AN;
  Result.Id := j;
  AN.Childs.Add(Result);
end;

initialization
  AdminDiv := TAdminNode.Create;
finalization
  AdminDiv.Destroy;
end.
