unit EccCSlav;

interface

uses
  Classes, EccDefs;

type
  TLexicName = class(TPersistent)
  public
    Num, Gen: Integer;
    Name: array[1..3] of string;
    Template: array[1..3] of Integer;
  end;

  TPhraseTemplate = record
    Num, Pad: Integer;
    Text: array[1..3] of string;
  end;

  TTemplateElement = record
    CutCount: Integer;
    AppendEnd: string;
    procedure Unpack(const S: string);
    function Pack: string;
  end;

  TLexicTemplate = record
    Elems: array[1..7] of TTemplateElement;
    Group: Integer;
  end;

  LexicArray = array of TLexicName;

function FirstCap(const s: string): string;
function FirstLength(const S: string): Integer;
function CSNum(I: Integer): string;
function End2Dot(const s: string): string;
procedure SortStrings(AStrings: TStrings);
function PosInStrings(const S: string; const List: TStrings): Integer;
procedure UnpackTemplate(S: string; var T: TLexicTemplate);
function PackTemplate(const T: TLexicTemplate): string; inline;
function ApplyTemplate(const S: string; Num, Index: Integer): string; overload;
function ApplyTemplate(const LN: TLexicName; Num, Index: Integer): string; overload;
function Translit(const S: string): string;
function LexCount(C: Integer): Integer;
function ExpandTitlo(S: string): string;
function UpperCaseCS(const S: string): string;
function LowerCaseCS(const S: string): string;
function TruncPray(const S: string): string;
function FindEmptyTemplate: Integer;
function CutPray(const s: string): string;

const
  _GlasSet: string = '0ABEHIJKMNOQSUWYZaehijkmnoqsuwyz{|';
  _UdarSet: string = '#$%''123456@^`~';
  _TitlSet: string = '&7<=>?C\bcdg';
  _TitbSet: string = '9DGLRlr}';
  _SoglSet: string = 'FPTVXfptvx';
  _SignSet: string = '8_';
  _PuncSet: string = '.,:!;/*[]()';

  _HoliSymbols: string = '* @@@$%^^^';

var
  LexicTemplates: array of TLexicTemplate;
  RolesList, GradesList, NamesList, LexicList, ParentsList,
    TownsList, HolidaysList, WordsList, IdentsList, ObjectsList,
    NumbersList, ModifsList: LexicArray;
  PripevsList: array of TPhraseTemplate;

implementation

uses
  SysUtils, EccProcs;

const
  _LCSym: string = 'ഃ糿onwqutkzjxpfvm12345c8';
  _UCSym: string = 'ƽǲONWQUTKZXPFVM~@#$%C_';
  _DiaSym: string ='123456c8~@#$%C_';

function Translit(const S: string): string;
const
  _lig: array[0..12] of array[0..1] of string =
    (('t', ''), ('x', ''), ('p', ''),
    ('V#', ''), ('V$', ''), ('v3', ''), ('v4', ''),
    ('', ''), ('', ''), ('', ''),
    ('e', 'e'), ('j', ''), ('e', 'e'));
  _end: array[0..3] of array[0..1] of string =
    (('', ''), ('', ''), ('', ''), ('', ''));
  _ends: string = ' ,.;!)]*/';
  _1: string = 'HB0aehijkmnqsyu|wzfv';
  _2: string = '';
var
  i, j: Integer;
begin
  if S.IsEmpty then Exit(S);
  Result := ExpandTitlo(S);
  for i := 0 to High(_lig) do
    Result := Result.Replace(_lig[i][0], _lig[i][1], [rfReplaceAll]);
  for i := 0 to 1 do
    Result := Result.Replace(_1.Chars[i], _2.Chars[i], [rfReplaceAll]);
  for i := 2 to _1.Length - 1 do
    Result := Result.Replace(_1.Chars[i], _2.Chars[i], [rfReplaceAll, rfIgnoreCase]);
  for i := 0 to _DiaSym.Length - 1 do
    Result := Result.Replace(_DiaSym.Chars[i], '', [rfReplaceAll]);
  for i := 0 to High(_end) do
    begin
      if Result.Substring(Result.Length - _end[i][0].Length) = _end[i][0] then
        Result := Result.Remove(Result.Length - _end[i][0].Length) + _end[i][1];
      for j := 0 to _ends.Length - 1 do
        Result := Result.Replace(_end[i][0] + _ends.Chars[j], _end[i][1] + _ends.Chars[j], [rfReplaceAll]);
    end;
end;

function CompareStringsCS(const S1, S2: string): Integer;
const
  _1: string = '0BHAEJSYoaehijkmnqsyu|8';
  _2: string = 'wzvwzzv峳zwzv';
  _sym: string = ' wtzxpfv';
var
  i, j, n, d, f, l1, l2: Integer;
  ch1, ch2: Char;
begin
  i := 0;
  j := 0;
  f := -1;
  d := -1;
  l1 := s1.Length - 1;
  l2 := s2.Length - 1;
  repeat
    if (j > l2) and (i > l1) then Exit(0);
    if (d > f) or (j > l2) then Exit(1);
    if (d < f) or (i > l1) then Exit(-1);
    f := _sym.IndexOf(S2.Chars[j]);
    while f = -1 do
      begin
        ch2 := S2.Chars[j];
        n := _1.IndexOf(ch2);
        if n >= 0 then ch2 := _2.Chars[n] else
          begin
            if j = l2 then Break;
            Inc(j);
            ch2 := S2.Chars[j];
          end;
        f := _sym.IndexOf(ch2);
      end;
    d := _sym.IndexOf(S1.Chars[i]);
    while d = -1 do
      begin
        ch1 := S1.Chars[i];
        n := _1.IndexOf(ch1);
        if n >= 0 then ch1 := _2.Chars[n] else
          begin
            if i = l1 then Break;
            Inc(i);
            ch1 := S1.Chars[i];
          end;
        d := _sym.IndexOf(ch1);
      end;
    Inc(i);
    Inc(j);
  until False;
end;

function StringListSortCompareCS(List: TStringList; Index1, Index2: Integer): Integer;
begin
  Result := CompareStringsCS(List[Index1], List[Index2]);
end;

procedure SortStrings(AStrings: TStrings);
var
  List: TStringList;
begin
  List := TStringList.Create;
  List.AddStrings(AStrings);
  List.CustomSort(StringListSortCompareCS);
  AStrings.Clear;
  AStrings.AddStrings(List);
  List.Free;
end;

function PosInStrings(const S: string; const List: TStrings): Integer;
var
  l, r: Integer;
begin
  l := 0;
  r := List.Count - 1;
  repeat
    Result := (l + r) div 2;
    if CompareStringsCS(S, List[Result]) < 0 then r := Result
    else l := Result;
  until (r - l) = 1;
  if CompareStringsCS(S, List[Result]) >= 0 then Result := l;
end;

function CSNum(I: Integer): string;
const
  n1: string = '亾fixopw';
  n2: string = ' G    }  l    R   t ';
var
  j, n: Byte;
  s: string;
begin
  Result := '';
  if (I < 1) or (I > 9999) then Exit;
  s := IntToStr(I);
  for j := 0 to 2 do
    begin
      if s = '' then Break;
      n := StrToInt(s[Length(s)]);
      SetLength(s, Length(s) - 1);
      if n = 0 then Continue;
      if (j = 1) and (n = 1) then  Result := Result + n1[n + j * 9]
      else Result := n1[n + j * 9] + Result;
    end;
  if s <> '' then
    begin
      n := StrToInt(s[1]);
      Result := '' + n1[n] + Result;
    end;
  n := Length(Result);
  if (n > 1) and not ((n = 2) and (Result[1] = '')) then Dec(n);
  j := Pos(Result[n], n1);
  if n2[j] <> #32 then Result[n] := n2[j]
  else Result := Copy(Result, 1, n) + '7' + Copy (Result, n + 1, Length(Result) - n);
end;

function UpperCaseCS(const S: string): string; {opt}
var
  i: Integer;
begin
  Result := S;
  for i := 0 to _LCSym.Length - 1 do
    Result := Result.Replace(_LCSym.Chars[i], _UCSym.Chars[i], [rfReplaceAll]);
end;

function LowerCaseCS(const S: string): string; {opt}
var
  i: Integer;
begin
  Result := S.Replace('#', '3', [rfReplaceAll]).Replace('$', '4', [rfReplaceAll]).Replace('%', '5', [rfReplaceAll]);
  for i := 0 to _UCSym.Length - 1 do
    Result := Result.Replace(_UCSym.Chars[i], _LCSym.Chars[i], [rfReplaceAll]);
end;

function FirstLength(const S: string): Integer; {opt}
begin
  if S.Length = 0 then Exit(0);
  Result := 1;
  while (S.Length >= Result + 1)
    and (S.Chars[Result - 1] in ['[', '*', '', '(', #32]) do Inc(Result);
  if S.Length >= (Result + 1) then
    if _DiaSym.IndexOf(S.Chars[Result]) >= 0 then Inc(Result);
end;

function FirstCap(const S: string): string; {opt}
var
  i: Integer;
begin
  i := FirstLength(S);
  if (i = 0) or ((S.Chars[0] = '[') and (S.Chars[2] = '[')) then Exit(S);
  Result := UpperCaseCS(S.Substring(0, i)) + S.Substring(i, MaxInt);
end;

function End2Dot(const S: string): string;
begin
  if (Length(S) = 0) or CharInSet(S.Chars[S.Length - 1], ['.', ':', ';', '!']) then Exit(S);
  if S.Chars[S.Length - 1] = ',' then
    Result := S.Substring(0, S.Length - 1) + ':'
  else Result := S + ':';
end;

procedure ModifyWord(var S: string);
const
  _shgl1: array[1..8] of string =
    ('1', '2', '', 'j', '', 'A', 'a', '');
  _shgl2: array[1..8] of string =
    ('h', '2', '', 'h', '', 'S', 's', 'z');
  _gl1: array[1..9] of string =
    ('2', '0', '', 'E', 'e', '', 'o1', '', 'o');
  _gl2: array[1..9] of string =
    ('w2', 'H', 'w', '2', '1', '', 'H', 'w4', 'w');
  _ac1: array[1..18] of string =
    ('A', 'a', 'Y', 'y', '2', '1', 'j', '2', 'h', '1', 'H', 'S', 's', '2', '1', '', '', '');
  _ac2: array[1..18] of string =
    ('', '', '{', '{', '6', '6', '', '6', '6', '6', 'w6', '', '', '6', '6', 'B', 'B', '');
var
  i, j: Integer;
  function _condrepl(var s: string; const s1, s2: string; index: Integer; sh: Boolean = False): Boolean;
  begin
    if sh and ((index <= 1) or ('9'.IndexOf(s.Chars[index - 2]) < 0)) then Exit(False);
    if s.Substring(index - 1, s1.Length) <> s1 then Exit(False);
    s := s.Substring(0, index - 1) + s2 + s.Substring(index + s1.Length - 1);
    Result := True;
  end;
begin
  for i := S.Length downto 1 do
    begin
      for j := 1 to High(_gl1) do
        if _condrepl(S, _gl1[j], _gl2[j], i) then Exit;
      if i <> S.Length then
        for j := 1 to High(_shgl1) do
          if _condrepl(S, _shgl1[j], _shgl2[j], i, True) then Exit;
      for j := 1 to High(_ac1) do
        if _condrepl(S, _ac1[j], _ac2[j], i) then Exit;
    end;
end;

function ApplyTemplate(const S: string; Num, Index: Integer): string; overload;
var
  ss: string;
begin
  StrDivide(S, Result, ss, '/');
  Result := Result.Substring(0, Result.Length
    - LexicTemplates[Num].Elems[Index].CutCount)
    + LexicTemplates[Num].Elems[Index].AppendEnd;
  if (Result <> '') and (Result.Chars[Result.Length - 1] = '*') then
    begin
      SetLength(Result, Result.Length - 1);
      ModifyWord(Result);
    end;
  Result := Result + ss;
end;

function ApplyTemplate(const LN: TLexicName; Num, Index: Integer): string; overload;
begin
  Result := ApplyTemplate(LN.Name[Num], LN.Template[Num], Index);
end;

function TTemplateElement.Pack: string;
begin
  Result := string.Create('-', CutCount) + AppendEnd;
end;

procedure TTemplateElement.Unpack(const S: string);
begin
  CutCount := 0;
  while (S.Length > CutCount) and (S.Chars[CutCount] = '-') do
    Inc(CutCount);
  AppendEnd := S.Substring(CutCount, MaxLongInt);
end;

procedure UnpackTemplate(S: string; var T: TLexicTemplate);
var
  j: Integer;
begin
  for j := 1 to 7 do
    begin
      StrDivide(S, T.Elems[j].AppendEnd, S, '/');
      T.Elems[j].Unpack(T.Elems[j].AppendEnd);
    end;
end;

function PackTemplate(const T: TLexicTemplate): string; inline;
var
  j: Integer;
begin
  Result := '';
  for j := 1 to 6 do
    Result := Result + T.Elems[j].Pack + '/';
  Result := Result + T.Elems[7].Pack;
end;

function LexCount(C: Integer): Integer;
begin
  case C of
    1: Result := 1;
    2: if _s_ForceDouble then Result := 2
      else Result := 3;
    else Result := 3;
  end;
end;

function ExpandTitlo(S: string): string;
var
  I: Integer;
const
  _tit: array[0..178] of array[0..1] of string =
    (('c', 'e'),
    ('c', ''),
    ('c', '0'),
    ('c2', '2'),
    ('c', 'e'),
    ('c', 'e') {},
    ('c', ''),
    ('c', ''),
    ('c', 'e'),
    ('c', '1'),
    ('c', 'e'),
    ('c', 'j'),
    ('c', 'j'),
    ('c', '0'),
    ('c', '0'),
    ('c', ''),
    ('c', 'e') {},
    ('c', ''),
    ('c', 'e'),
    ('c', '1'),
    ('c', 'e'),
    ('c', ''),
    ('c', ''),
    ('c', 'z'),
    ('c', ''),
    ('c', 'e'),
    ('c', 'e'),
    ('c', 'e'),
    ('c', 'e'),
    ('c', '1'),
    ('c', 'e'),
    ('c', 'e'),
    ('c', 'e'),

    ('G', '0') {},
    ('G', ''),
    ('G', ''),

    ('l', 'a') {},
    ('l', ''),
    ('l', 'a') {},
    ('l', ''),
    ('l', 'h'),
    ('l', 'h'),
    ('l', ''),
    ('l', '0'),
    ('l', '1'),
    ('l', '0'),
    ('l', 'a'),
    ('l', '0'),
    ('l', ''),
    ('l', ''),

    ('L', 'h'),
    ('L', 'h'),
    ('L', 'h'),
    ('L', 'h'),
    ('L', 'e'),
    ('L', 'e'),
    ('L', ''),

    ('d', 'a'),
    ('d', 'y'),
    ('d', '0'),
    ('d', '0'),
    ('d', ''),

    ('', ''),
    ('', 'e'),

    ('7', 'e'),
    ('7', ''),
    ('7', ''),
    ('7', 'e'),
    ('7', ''),
    ('7', 'j'),
    ('71', '1'),
    ('7', ''),
    ('7', '0'),
    ('7', ''),
    ('7', 'z'),
    ('7', 'e'),
    ('7', 'e'),
    ('7', 's'),
    ('7', 'z'),
    ('7', 'h'),
    ('7', 'h'),
    ('7', ''),
    ('7', 'a'),
    ('7', 'a'),
    ('7', 'a'),
    ('7', 'a'),
    ('7', 'a'),
    ('7', 'a'),
    ('7', '') {},
    ('7', 'e'),
    ('7', ''),
    ('7', 'y') {},
    ('7', ''),
    ('7', 'a'),
    ('7', 'z'),
    ('7', 'z'),

    ('}', 'y'),
    ('}', 'a'),

    ('9e', 'e') {},
    ('9', '0'),
    ('9', ''),

    ('', ''),
    ('', 'y'),
    ('', 'y'),
    ('', 'y'),
    ('', ''),
    ('n', 'ne'),
    ('', ''),
    ('', '1'),
    ('', '1'),

    ('R', ''),
    ('Rh', 'h'),
    ('R', 'e'),
    ('R', '') {},
    ('Rj', 'j'),
    ('R', 'e'),
    ('R', 'e'),
    ('R', 'e'),
    ('R', 'e'),
    ('R', 'a') {},
    ('R', ''),
    ('R', ''),

    ('r', ''),
    ('r', 'e'),
    ('r2', '2'),
    ('r', ''),
    ('r', ''),
    ('r', 'e'),
    ('r', '1'),
    ('r', 'e'),
    ('r', 'e'),
    ('r', ''),
    ('r', '1'),
    ('r', ''),
    ('r', '') {},
    ('re', 'ae'),
    ('r', 'a'),
    ('r', 'a'),
    ('r', 's'),
    ('r', '0'),
    ('r', '0'),
    ('r', '0'),
    ('r', '0'),
    ('r', 'z'),
    ('r', ''),
    ('r', 'a'),

    ('D', ''),
    ('D1', '1'),
    ('D', 'a'),
    ('D', '0'),
    ('D', '0'),
    ('D', '0'),
    ('D', '0'),
    ('D', '0'),
    ('D', '0'),

    ('0', '0') {},
    ('', 'y'),
    ('', 'y'),
    ('g', 'a'),
    ('+', ''),
    ('>', 'z'),
    ('8', ''),
    ('j', 'vj'),
    ('', 'va'),

    ('', 'a'),
    ('', 'a'),
    ('', 'z1'),
    ('', 'z1'),
    ('', 's'),
    ('', 's'),
    ('', 's'),
    ('', 's'),
    ('', 's'),
    ('', 's'),
    ('', 'z'),

    ('b', '0'),
    ('b', '0'),
    ('b', ''),
    ('b', '0'));
begin
  Result := S;
  for I := 0 to High(_tit) do
    Result := Result.Replace(_tit[I][0], _tit[I][1], [rfReplaceAll]);
end;

function TruncPray(const S: string): string;
var
  i: Integer;
begin
  Result := S.Substring(0, S.Substring(0, 26).LastIndexOf(#32));
  i := Result.LastIndexOfAny([',', ';', '.']);
  if i >= (Result.Length - 10) then Result := Result.Substring(0, i)
  else
    begin
      i := Result.LastIndexOf(#32);
      if i >= (Result.Length - 4) then Result := Result.Substring(0, i);
    end;
  while _PuncSet.IndexOf(Result.Chars[Result.Length - 1]) >= 0 do Result := Result.Substring(0, Result.Length - 1);
  Result := Result + ':';
end;

function FindEmptyTemplate: Integer;
begin
  for Result := 1 to High(LexicTemplates) do
    if (LexicTemplates[Result].Elems[2].CutCount = 0) and (LexicTemplates[Result].Elems[2].AppendEnd = '') then
      Exit;
  Result := -1;
end;

function CutPray(const s: string): string;
begin
  Result := s.Substring(0, 32).Replace(#10, #32);
end;

end.


