﻿unit EccCalend;

interface

var
  Y_D, M_D, D_D, DW_D: Word;

function IsLeap(Y: Integer; Greg: Boolean): Boolean;
procedure CalendarConvert(var Y, M, D: Integer; ToJuly: Boolean; OurAge: Boolean = True);
procedure SetCalendar(NDate: TDateTime);
function CalcAssembly: Integer;

implementation
uses
  SysUtils, DateUtils, StrUtils, UITypes, Generics.Collections,
  FMX.StdCtrls,   FMX.SpinBox, FMX.ListBox, FMX.Dialogs,
  FMX.Controls, FMX.Types, EcclysiarchMain, EccDefs, EccVars,
  EccProcs, EccCSlav, EccPrays, EccScripts, EccMemos, EccLists;

function IsLeap(Y: Integer; Greg: Boolean): Boolean;
begin
  Result := ((Y mod 4) = 0);
  if Greg then
    Result := (Result and not ((Y mod 100) = 0)) or ((Y mod 400) = 0);
end;

procedure CalendarConvert(var Y, M, D: Integer; ToJuly: Boolean; OurAge: Boolean = True);
var
  DInc: Integer;
  Vs: Boolean;
begin
  DInc := Trunc(Y/100);
  if not OurAge and (DInc <> 0) then Dec(DInc);
  DInc := (DInc div 4) * 3 + (DInc mod 4);
  if OurAge then Dec(DInc, 2)
  else
    begin
      Inc(DInc, 2);
      DInc := -DInc;
    end;
  if IsLeap(Y, True) and not IsLeap(Y, False) and (M < 3) then
    if OurAge then Dec(DInc) else Inc(DInc);
  if ToJuly then D := D - DInc else D := D + DInc;
  Vs := IsLeap(Y, ToJuly);
  while D > MonthDays[Vs][M] do
    begin
      D := D - MonthDays[Vs][M];
      Inc(M);
      if M > 12 then
      begin
        Inc(Y);
        M := 1;
        Vs := IsLeap(Y, ToJuly);
      end;
    end;
  while D <= 0 do
    begin
      if M = 1 then
        begin
          M := 12;
          Dec(Y);
          Vs := IsLeap(Y, ToJuly);
        end
      else Dec(M);
      D := MonthDays[Vs][M] + D;
    end;
end;

function Easter(Y: Integer): TDateTime;
var
  d, m: Integer;
begin
  d := (2 * (Y mod 4) + 4 * (Y mod 7) + 6 * ((19 * (Y mod 19) + 15) mod 30 + 1)) mod 7 + (19 * (Y mod 19) + 15) mod 30 + 22;
  if d > 31 then
    begin
      m := 4;
      d := d - 31;
    end
  else m := 3;
  CalendarConvert(Y, m, d, False);
  Result := EncodeDate(Y, m, d);
end;

var
  _arranged: TList<Integer>;

procedure AddMemExemplar(const Mem: TMemory; DT: TDateTime; A: Boolean = True);
var
  m: TMemExemplar;
  dr: TDateRec;
begin
  if not DatesDic.ContainsKey(DT) then
    begin
      dr.Rank := 0;
      dr.MemList := TList<TMemExemplar>.Create;
      DatesDic.Add(DT, dr);
    end
  else dr := DatesDic[DT];
  for m in dr.MemList do if m.Memory = Mem then Exit;
  m := TMemExemplar.Create;
  m.Memory := Mem;
  m.Rank := Mem.Rank;
  m.Active := Mem.Default and A;
  case Length(Mem.Services) of
    0: m.ItemIndex := -1;
    else m.ItemIndex := 0;
  end;
  dr.MemList.Add(m);
  if m.Active and (m.Rank > dr.Rank) then dr.Rank := m.Rank;
  DatesDic[DT] := dr;
end;

function NeedOctoih(i: Integer): Boolean;
begin
  try
    Result := ((i mod 7 = 1) and (DatesDic[i].Rank < 9)) or (DatesDic[i].Rank < 4);
  except
    Result := True;
  end;
end;

procedure ArrangeMemories(Y: Integer);
var
  i, j, k, m, d, yy: Integer;
  mem: TMemory;
  dt, p, c: TDateTime;
const
  _sh1 = 169;
  _sh2 = 281;

  procedure _dateProc;
  begin
    mem := MineaList[i];
    m := mem.Date.M;
    d := mem.Date.D;
    yy := Y;
    CalendarConvert(Y, m, d, False);
    dt := EncodeDate(Y, m, d);
    Y := yy;
  end;

begin
// распределяем существующие записи <date> по дням
  for i := 0 to MineaList.Count - 1 do
    if MineaList[i].ExecNode = nil then
      begin
        _dateProc;
        AddMemExemplar(mem, dt);
      end;
  for i := 0 to MineaList.Count - 1 do
    if MineaList[i].ExecNode <> nil then
      begin
        _dateProc;
        ClearVarGroup('Work');
        Vars.AddOrSetValue('Work.DayOfWeek', DayOfWeek(dt).ToString);
        ProcessChild(mem.ExecNode, _notParam, nil);
        dt := IncDay(dt, GetIntVar('Work.Date.Offset'));
        AddMemExemplar(mem, dt);
      end;
// распределяем чинопоследования Триоди
  m := 9; d := 14; yy := Y;
  CalendarConvert(yy, m, d, False);
  c := Easter(yy);
  k := Trunc(StartOfTheWeek(IncWeek(EncodeDate(yy, m, d))) - c);
  j := Trunc(Easter(yy + 1) - 69 - c);
  for i := 0 to TriodList.Count - 1 do
    if TriodList[i].ExecNode = nil then
      begin
        mem := TriodList[i];
        if mem.Date.D < j then // крещенская отступка
          begin
            AddMemExemplar(mem, IncDay(c, mem.Date.D));
            if (j - _sh2) >= (_sh2 - mem.Date.D) then
              AddMemExemplar(mem, IncDay(c, mem.Date.D + j - _sh2));
          end
        else if (mem.Date.D < _sh1) and (mem.Date.D < k) then
          begin                // воздвиженская отступка
            AddMemExemplar(mem, IncDay(c, mem.Date.D));
            if (k - _sh1) >= (_sh1 - mem.Date.D) then
              AddMemExemplar(mem, IncDay(c, mem.Date.D + k - _sh1));
          end
        else if mem.Date.D >= _sh1 then
          AddMemExemplar(mem, IncDay(c, mem.Date.D + k - _sh1));
      end;
  for i := 0 to TriodList.Count - 1 do
    if TriodList[i].ExecNode <> nil then
      begin
        mem := TriodList[i];
        dt := IncDay(c, TriodList[i].Date.D);
        ClearVarGroup('Work');
        Vars.AddOrSetValue('Work.DayOfWeek', DayOfWeek(dt).ToString);
        ProcessChild(mem.ExecNode, _notParam, nil);
        AddMemExemplar(mem, IncDay(dt, GetIntVar('Work.Date.Offset')));
      end;
// распределяем чинопоследования Октоиха
  p := IncDay(Easter(Y + 1), -8);
  dt := IncDay(c, 7);
  for i := 0 to 6 do AddMemExemplar(OctList[(i + Ord((i = 6) and not GetFlag('Variants.SvetSubbGreekVoice'))) * 7], c + i);
  for i := -7 to -1 do AddMemExemplar(OctList[56 + i], c + i, False);
  for i := Trunc(dt) to Trunc(p) do
    AddMemExemplar(OctList[DaysBetween(dt, i) mod 56], i, NeedOctoih(i));
  _arranged.Add(Y);
end;

procedure SetCalendar(NDate: TDateTime);
var
  Y, M, D, mm, i, j: Integer;
  DW, e: Byte;
  v: Boolean;
  s: string;
  mr: TMemExemplar;
  dt: TDateTime;
  item: TCheckBox;
  sgn: TLabel;
  function _findTriod(dt: TDateTime): TMemExemplar;
  begin
    if DatesDic.ContainsKey(dt) then
      for Result in DatesDic[dt].MemList do if Result.Memory.Source = 3 then Exit;
  end;
  function _getPeriod(w, dw: Integer): Integer;
  begin
    Result := 1;
    case w of
      -10: if dw = 1 then Result := 0;
      -9: if dw = 1 then Result := 0 else if dw = 6 then Result := 4;
      -8: if dw <> 1 then Result := 3 else Result := 0;
      -7..-1: Result := 0;
      0..4: Result := 2;
      5: if dw < 5 then Result := 2;
      6: if dw = 6 then Result := 4;
    end;
  end;
  procedure _setCalendVars;
  var
    vr: TPair<string, string>;
  begin
    Vars.AddOrSetValue('Today.Year', Y.ToString);
    Vars.AddOrSetValue('Today.Month', M.ToString);
    Vars.AddOrSetValue('Today.Day', D.ToString);
    Vars.AddOrSetValue('Today.Date', (M * 100 + D).ToString);
    Vars.AddOrSetValue('Today.DayOfWeek', DW.ToString);
    Vars.AddOrSetValue('Today.DayOfWeek.Code', ShortDWs[DW]);
    dt := Easter(Y - 1);
    Vars.AddOrSetValue('Today.Day.Past', Trunc(NDate - dt).ToString);
    i := Trunc(NDate - dt + 700) div 7 - 100;
    Vars.AddOrSetValue('Today.Week.Past', i.ToString);
    Vars.AddOrSetValue('Today.Week.Voice', ((i - 1) mod 8 + 1).ToString);
    dt := Easter(Y);
    Vars.AddOrSetValue('Today.Day.Current', Trunc(NDate - dt).ToString);
    i := Trunc(NDate - dt + 700) div 7 - 100;
    Vars.AddOrSetValue('Today.Week.Current', i.ToString);
    if i > 0 then
      Vars.AddOrSetValue('Today.Week.Voice', ((i - 1) mod 8 + 1).ToString)
    else if i > -2 then
      Vars.AddOrSetValue('Today.Week.Voice', '0');
    Vars.AddOrSetValue('Today.Week.Current', i.ToString);
    Vars.AddOrSetValue('Today.Year.IsLeap', Ord(IsLeap(Y, True)).ToString);
    Vars.AddOrSetValue('Today.Period', _getPeriod(i, DW).ToString);
    for vr in Vars do
      if (vr.Key.Substring(0, 5) = 'Temp.') or (vr.Key = 'Temp') or
        (vr.Key.Substring(0, 7) = 'Service') then Vars.Remove(vr.Key);
  end;
  function _hasArranged(Y: Integer): Boolean;
  begin
    Result := False;
    for var i in _arranged do if i = Y then Exit(True);
  end;
begin
  _OnModify := True;
// очищаем спидбаттоны календаря
  for i := Low(CalendButtons) to High(CalendButtons) do
    with CalendButtons[i] do
      begin
        Text := '';
        Hint := '';
        IsPressed := False;
        Opacity := 1;
      end;
// настраиваем переменные
  DecodeDate(NDate, Y_D, M_D, D_D);
  DW_D := DayOfWeek(NDate);
  Y := Y_D;
  M := M_D;
  D := D_D;
  DW := DW_D;
  CalendarConvert(Y, M, D, True);
  _setCalendVars;
  if EccMainForm.ComboBox2.ItemIndex = 1 then CalendarConvert(Y, M, D, False);
  EccMainForm.Label1.Text := D.ToString;
  EccMainForm.SpinBox1.Value := Y;
  EccMainForm.ComboBox1.ItemIndex := M - 1;
  DW := (DW + 34 - D) mod 7 + 1;
  v := IsLeap(Y, EccMainForm.ComboBox2.ItemIndex = 0);
  if M > 1 then mm := MonthDays[v][M - 1] else mm := MonthDays[v][12];
  if not _hasArranged(Y - 1) then ArrangeMemories(Y - 1);
  if not _hasArranged(Y) then ArrangeMemories(Y);
  ProcessChild(ProcDic.Items['year_init'], _notParam, PrayList);
// заполняем спидбаттоны
  dt := NDate - D - DW;
  for i := 1 to 42 do
    with CalendButtons[i] do
      begin
        Tag := Trunc(IncDay(dt, i));
        {$IFDEF DEBUG}
        ShowHint := True;
        Hint := FormatDateTime('d/m/y', Tag);
        {$ENDIF}
        if Vars[SysVars[41]] = '1' then
          begin
            j := DatesDic[Tag].Rank;
            case j of
              0: FontColor := $FFAF2FAF;
              1..2: FontColor := $FF9F9F9F;
              3..4: FontColor := $FFAFAFAF;
              5: FontColor := $FFCF8F8F;
              6: FontColor := $FFEF5F5F;
              7..9: FontColor := $FFFF2F2F;
            end;
            if j > 7 then Font.Style := [TFontStyle.fsBold]
            else Font.Style := [];
            Font.Size := 12 + j;
          end
        else
          begin
            FontColor := $FFAFAFAF;
            Font.Style := [];
            Font.Size := 16;
          end;
      end;
  for i := 1 to DW do
    with CalendButtons[i] do
      begin
        Text := IntToStr(mm - DW + i);
        Opacity := 0.3;
      end;
  for i := 1 to MonthDays[v][M] do
    CalendButtons[i + DW].Text := IntToStr(i);
  for i := 1 to 42 - DW - MonthDays[v][M] do
    with CalendButtons[i + DW + MonthDays[v][M]] do
      begin
        Text := IntToStr(i);
        Opacity := 0.3;
      end;
// нумеруем недели
  dt := IncDay(NDate, -6 - D - DW);
  for e := 1 to 6 do
    begin
      dt := IncDay(dt, 7);
      mr := _findTriod(dt);
      i := (mr.Memory.Date.D + 70) div 7 - 9;
      s := 'седмица ';
      case i of
        -9..-8: s := s + (i + 10).ToString + '-я подготовительная';
        -7: s := s + 'сырная';
        -6..-1: s := s + (i + 7).ToString + '-я поста';
        0: s := s + 'страстная';
        1: s := s + 'свѣтлая';
        2..7: s := s + i.ToString + '-я по Пасхѣ';
        else s := s + (i - 7).ToString + '-я по Пятидесятницѣ';
      end;
      WeekLab[e].Text := s;
      if Vars[SysVars[41]] = '1' then
        case i of
          1..8: WeekLab[e].FontColor := $FFBF8F8F;
          -6..0: WeekLab[e].FontColor := $FFAF7FAF;
          -7: WeekLab[e].FontColor := $EFBFBF00;
          else WeekLab[e].FontColor := $FF9F9F9F;
        end
      else WeekLab[e].FontColor := $FFAFAFAF;
    end;
// выбираем текущий день
  with CalendButtons[D + DW] do IsPressed := True;
// чистим списки памятей дня
  for i := 1 to 5 do
    with Expds[i] do
      begin
        while Controls[Expds[i].ControlsCount - 1].ControlsCount > 0 do
          Controls[Expds[i].ControlsCount - 1].Controls[0].Free;
        Height := 30;
      end;
// заполняем новыми чекбоксами
  for mr in DatesDic[NDate].MemList do
    begin
      e := mr.Memory.Source;
      item := TCheckBox.Create(Expds[e]);
      sgn := TLabel.Create(Expds[e]);
      with item do
        begin
          StyledSettings := [];
          Font.Family := 'Orthodox.tt ieUcs8';
          Font.Size := 24;
          if mr.Memory.Area = '' then
            if mr.Rank > 5 then
              FontColor := $ffff0000
            else
              FontColor := $ffafafaf
          else
            if mr.Rank > 5 then
              FontColor := $ffdf3f7f
            else
              FontColor := $ff9f9fcf;
          if Length(mr.Memory.Services) = 0 then Opacity := 0.75
          else Opacity := 1;
          Width := EccMainForm.VertScrollBox1.ClientWidth - 10;
          Height := 30;
          Anchors := Expds[e].Anchors;
          if ExactsList[mr.Memory.Exact] = '' then Text := mr.Memory.Title
          else Text := '[' + ExactsList[mr.Memory.Exact] + '] ' + mr.Memory.Title;
          IsChecked := mr.Active;
          TagObject := sgn;
          OnChange := EccMainForm.SelectMemory;
          OnMouseDown := EccMainForm.CBMouseDown;
          OnGesture := EccMainForm.CBGesture;
          Align := TAlignLayout.Top;
          Margins.Left := 32;
          Position.Y := MaxLongInt;
        end;
      with Expds[e] do
        begin
          v := IsExpanded;
          IsExpanded := True;
          AddObject(item);
          Height := item.Position.Y + item.Height + 30;
          AddObject(sgn);
          IsExpanded := v;
        end;
      with sgn do
        begin
          StyledSettings := [];
          Font.Family := 'Orthodox.tt eRoos';
          Font.Size := 32;
          Height := 28;
          Width := 28;
          if mr.Rank > 2 then FontColor := $ffff0000
          else FontColor := $ff000000;
          if Length(mr.Memory.Services) = 0 then Opacity := 0.75
          else Opacity := 1;
          Position.X := 5;
          Position.Y := item.Position.Y;
          Text := _HoliSymbols.Chars[mr.Rank + 1];
          HitTest := True;
          TagObject := TObject(mr);
          OnMouseDown := EccMainForm.CBMouseDown;
          OnGesture := EccMainForm.CBGesture;
          OnTap := EccMainForm.CBTap;
        end;
    end;
  _OnModify := False;
  EccMainForm.SelectMemory(nil);
  ProcessChild(ProcDic.Items['day_init'], _notParam, PrayList);
end;

function CalcAssembly: Integer;
var
  sr: TService;
  a: TServiceList;
  m: TMemExemplar;
  i, j: Integer;

  function _isLess(const S1, S2: TService): Boolean;
  var
    i, j: Integer;
  begin
    Result := False;
    i := IndexStr(S1.GloryId, _PredvarGlories);
    j := IndexStr(S2.GloryId, _PredvarGlories);
    if (j >= 0) and (i = -1) then Exit(True);
    if (i >= 0) and (j = -1) then Exit(False);
    if (i = -1) and (j = -1) then
      begin
        if S1.Rank < S2.Rank then Exit(True);
        if S1.Rank > S2.Rank then Exit(False);
        if S1.Rank = S2.Rank then Exit((S1.SortIdx < S2.SortIdx) and (S1.SortIdx <= 3));
{and <= 3 добавлено, чтобы только ангельские и предтечевы службы
всегда предваряли, а апостольская могла быть после святительской}
      end
    else Exit(i < j);
  end;

  procedure _adjustVars(i: Integer);
  var
    n: string;
    g: TGlory;
  begin
    n := 'Service' + i.ToString + '.';
    g := GloryDic[ServMap[i].GloryId];
    Vars.AddOrSetValue(n + 'Id', ServMap[i].Id);
    Vars.AddOrSetValue(n + 'Glory', ServMap[i].GloryId);
    Vars.AddOrSetValue(n + 'Grade', g.Grade.ToString);
    Vars.AddOrSetValue(n + 'Stared', g.Stared.ToString);
    Vars.AddOrSetValue(n + 'Holi', ServMap[i].Holi.ToString);
    Vars.AddOrSetValue(n + 'Title', ServMap[i].Title);
    Vars.AddOrSetValue(n + 'Rank', ServMap[i].Rank.ToString);
    Vars.AddOrSetValue(n + 'MioId', ServMap[i].MiOId);
    Vars.AddOrSetValue(n + 'IsSanct', Ord(g.TypeNum = 7).ToString);
    Vars.AddOrSetValue(n + 'IsSingle', Ord((g.TypeNum = 7) and (g.Stared = 1)).ToString);
    Vars.AddOrSetValue(n + 'IsSimple', Ord((g.TypeNum = 7) and not (g.Grade in [3, 4, 22, 25, 26])).ToString);
    Vars.AddOrSetValue(n + 'IsGospod', Ord(g.TypeNum <= 5).ToString);
    Vars.AddOrSetValue(n + 'IsBogorod', Ord(g.TypeNum = 6).ToString);
    Vars.AddOrSetValue(n + 'IsMovable', Ord(ServMap[i].IsMov).ToString);
    Vars.AddOrSetValue(n + 'IsHram', Ord(ServMap[i].Exact = 6).ToString);
  end;

  function _assigned(i: Integer): Boolean;
  begin
    Result := ServMap[i].Id <> '';
  end;

  function _getRank(i: Integer): Integer;
  begin
    if _assigned(i) then Result := ServMap[i].Rank
    else Result := -1;
  end;

  function _isSanct(i: Integer): Boolean;
  begin
    Result := _assigned(i) and GloryDic[ServMap[i].GloryId].IsSanct;
  end;

  function _maxTodayRank: Integer;
  var
    i, j: Integer;
  begin
    Result := -1;
    for i := 1 to 7 do
      begin
        j := _getRank(i);
        if Result < j then Result := j;
      end;
  end;

  function _findFree: Integer;
  begin
    Result := -1;
    for var i := 4 to 7 do if not _assigned(i) then Exit(i);
  end;

  procedure _setSanctList;
  var
    b: Boolean;
    s: string;
    i: Integer;
  begin
    b := False;
    s := '';
    for i := 3 to 8 do
      if _isSanct(i) then
        begin
          b := True;
          s := s + ServMap[i].GloryId + ', ';
        end;
    Vars.AddOrSetValue('Today.Glory.HasSanct', Ord(b).ToString);
    Vars.AddOrSetValue('Today.Glory.List', s.Substring(0, s.Length - 2));
  end;

  procedure _fin(n: Integer);
  begin
    Result := n;
    Abort;
  end;

begin
  try
    ResetServMap;
    a := TServiceList.Create;
    // все активные последования распределяем в карту
    for m in DatesDic[EncodeDate(Y_D, M_D, D_D)].MemList do
      if m.Active and (m.Memory.Exact <> 8) then
        begin
          with sr do
            begin
              Rank := m.Rank;
              SortIdx := SanctLexList[m.Memory.Glory.Grade].FSort;
              Exact := m.Memory.Exact;
              if Length(m.Memory.Services) > 0 then Id := m.Memory.Services[0].Id
              else Id := 'mi_o.' + m.Memory.Glory.MiOId;
              GloryId := m.Memory.Glory.Id;
              Holi := m.Memory.Holi;
              Relic := m.Memory.Relic;
              Title := m.Memory.Title;
              IsMov := (m.Memory.Exact = 7) and (m.Memory.Source = 1);
            end;
          // слот 1, если октоих
          if sr.Exact = 9 then
            begin
              if _assigned(1) then _fin(-15);
              ServMap[1] := sr;
              Continue;
            end;
          // слот 4, если храм
          if sr.Exact = 6 then
            begin
              ServMap[4] := sr;
              Continue;
            end;
          // слот 7, если Пасха, попразднство, отдание
          if sr.GloryId = _PredvarGlories[0] then
            begin
              ServMap[7] := sr;
              Continue;
            end;
          // слот 2, если пред/попразднство двунадесятого
          if sr.Exact in [1..5] then
            begin
              if _assigned(1) then _fin(-16);
              ServMap[2] := sr;
              Continue;
            end;
          // слот 3, если триодь или малое пред/попразднство
          if sr.Exact in [7, 10..12] then
            begin
              if _assigned(1) then _fin(-17);
              ServMap[3] := sr;
              Continue;
            end;
          // слот 8, если на повечерие
          if sr.Rank < 0 then
            begin
              if _assigned(1) then _fin(-18);
              ServMap[8] := sr;
              ServMap[8].Rank := 0;
              Continue;
            end;
          a.Add(sr);
        end;
    // сортируем оставшиеся последования
    for i := 2 to a.Count do
      for j := i - 1 downto 1 do
        if _isLess(a[j - 1], a[j]) then a.Exchange(j, j - 1)
        else Break;
    // и тоже распределяем в карту
    for i := 0 to a.Count - 1 do
      begin
        j := _findFree;
        if j < 0 then _fin(-6);
        ServMap[j] := a[i];
      end;
    // отказ, если неверные параметры
    j := 0;
    for i := 1 to 7 do
      if _assigned(i) then Inc(j);
    if j = 0 then _fin(-1); // не выбрано ни одного последования?
    // настраиваем переменные
    _setSanctList;
    j := 0;
    for i := 1 to 7 do if _getRank(i) > j then j := _getRank(i);
    Vars.AddOrSetValue('Today.Rank', j.ToString);
    Vars.AddOrSetValue('Today.HasOctoih', Ord(_assigned(1)).ToString);
    Vars.AddOrSetValue('Today.HasPolyeley', Ord(j >= 5).ToString);
    Vars.AddOrSetValue('Today.VelSlavosl', Ord((j >= 3) and (Vars['Today.Period'] <> '0')) .ToString);
    if (j >= 6) and _assigned(8) then _fin(-5);
    if _assigned(6) then _fin(-6);
    // ищем подходящую сборку
    if ServMap[4].Rank = 9 then // двунадесятый господский?
      begin
        if _assigned(1) then _fin(-7);
        if _assigned(6) then _fin(-10);
        case _getRank(5) of
          6..8: _fin(18);
          -1: _fin(13);
          else _fin(-10);
        end;
      end;
    if DW_D = 1 then // воскресение?
      begin
        if not _assigned(1) then // отсутствует Октоих?
          if _getRank(4) < 4 then _fin(-2) else _fin(-3);
        if not ServMap[1].Rank in [5, 6] then _fin(-7);
        if _assigned(4) then // присутствует основная
          begin
            if (_getRank(4) = 9) or (_getRank(5) = 9) then _fin(-11);
            if _assigned(3) then // есть Триодь: 22, 23 или -13
              if _assigned(5) then _fin(-13) //? воскресная + триодь + 2 основных с предпразднством или без
              else _fin(22 + Ord(_assigned(2)))
            else // нет Триоди: 11, 12, 24 или 25
              _fin(11 + 13 * Ord(_assigned(2)) + Ord(_assigned(5)));
          end;
           // отсутствует основная
        if _assigned(3) then // есть Триодь
          if not _assigned(2) then _fin(-13) //? воскресная + триодь без основного
          else
            if ServMap[2].Exact in [5] then _fin(21) // отдание
            else _fin(-13) //? воскресная + триодь с попразднством без основного
        else // нет Триоди
          if not _assigned(2) then _fin(2)
          else
            if ServMap[2].Exact in [5] then _fin(26) // отдание
            else _fin(-13); //? воскресная с попразднством без основного
      end;
    if _assigned(1) then // есть Октоих
      begin
        if ServMap[1].Rank > 0 then _fin(-8);
        if j >= 4 then _fin(-9);
        if _assigned(2) then _fin(-14);
        if DW_D = 7 then // суббота
          if _assigned(3) then // есть Триодь
            if _assigned(4) then
              if _assigned(5) then
                if j > 0 then _fin(28) else _fin(-19)
              else
                if j > 0 then _fin(29) else _fin(30)
            else _fin(-13)
          else      // нет Триоди
            if _assigned(4) then
              if _assigned(5) then
                if j > 0 then _fin(10) else _fin(-12)
              else
                if j > 0 then _fin(9) else _fin(8)
            else _fin(1)
        else // будний день
          if _assigned(3) then // есть Триодь (сырная)
            if _assigned(4) then
              if _assigned(5) then _fin(34) else _fin(33)
            else _fin(-13)
          else       // нет Триоди
            if _assigned(4) then
              if _assigned(5) then
                if j > 0 then _fin(7) else _fin(20)
              else
                if j > 0 then _fin(6) else _fin(5)
            else _fin(0);
      end;
    if _assigned(3) then // есть Триодь
      if not _assigned(2) then // нет попразднства
        if _assigned(4) then
          if _assigned(5) then
            if j >= 4 then _fin(-13) else _fin(27)
          else
            if j >= 4 then _fin(31) else _fin(32)
        else _fin(-13)
      else // есть попразднство
        if _assigned(4) then
          if _assigned(5) then _fin(-13)
          else
            if j >= 4 then _fin(-13) else _fin(35)
        else
          if ServMap[2].Exact = 6 then _fin(-13)
          else _fin(-13);
        // нет Триоди
    if not _assigned(2) then // нет попразднства
      if j <= 3 then _fin(-2)
      else
        if _assigned(4) then
          if _assigned(5) then _fin(18) else _fin(13)
        else _fin(-13)
    else // есть попразднство
      if _assigned(4) then
        if _assigned(5) then _fin(16) else _fin(15)
      else
        if ServMap[2].Exact = 6 then _fin(36)
        else _fin(-13);
  except
    if Result >= 0 then
      begin
        for i := 1 to 9 do _adjustVars(i);
        Vars.AddOrSetValue('Today.IsZaupokoy', Ord(Result in [1, 8, 14, 19]).ToString);
        Vars.AddOrSetValue('System.Assem', Result.ToString);
      end;
    a.Free;
  end;
end;

initialization
  OctList := TMemoriesList.Create;
  _arranged := TList<Integer>.Create;
finalization
  OctList.Free;
  _arranged.Free;
end.
