unit EccLists;

interface

uses
  Generics.Collections, Types, XmlIntf, EccCSlav, EccMemos;

type
  TPrayReplace = record
    Id, Val: string;
    Nomin: Integer;
  end;

  TGroupRec = record
    Id: Integer;
    Val, Title, Pre: string;
    PrayType: Integer;
  end;
  PGroupRec = ^TGroupRec;

var
  TypesList, StylesList, CountsList, VoicesList, SubvoicesList,
    OrdersList, AuthorsList, BooksList, RelicsList, ExactsList,
    AssemsList, ErrorsList, LangsList, TrapezasList: TStringDynArray;
  ReplList: array of TPrayReplace;
  GroupDic: TDictionary<string, TGroupRec>;
  GroupsList: array of TGroupRec;

const
  _LexicSym: string = 'GNLRPTHWICOM';
  MaxLists = 30;
  ListNames: array[0..MaxLists] of string =
    ('grades', 'names', 'lexic', 'roles', 'parents', 'towns',
    'holidays', 'words', 'idents', 'numbers', 'objects', 'modifs',
    'counts', 'types', 'styles', 'voices', 'subvoices', 'relics',
    'exacts', 'authors', 'books', 'orders', 'groups', 'sanctlex',
    'pripevs', 'templates', 'assems', 'errors', 'trapezas',
    'languages', 'replaces');
  ListArrays: array[0..MaxLists] of Pointer =
    (@GradesList, @NamesList, @LexicList, @RolesList, @ParentsList, @TownsList,
    @HolidaysList, @WordsList, @IdentsList, @NumbersList, @ObjectsList, @ModifsList,
    @CountsList, @TypesList, @StylesList, @VoicesList, @SubvoicesList, @RelicsList,
    @ExactsList, @AuthorsList, @BooksList, @OrdersList, @GroupsList, @SanctLexList,
    @PripevsList, @LexicTemplates, @AssemsList, @ErrorsList, @TrapezasList,
    @LangsList, @ReplList);

procedure LoadList(const Node: IXmlNode);
function FindStringArray(const S: string): Pointer;
function GetLexicArray(I: Integer): LexicArray; overload;
function GetLexicArray(Ch: Char): LexicArray; overload;

implementation
uses
  SysUtils, StrUtils, XMLDoc,
  EccProcs, EccXml, EccScripts;

function FindStringArray(const S: string): Pointer;
var
  i: Integer;
begin
  i := IndexStr(s, ListNames);
  if i >= 0 then Result := ListArrays[i] else Result := nil;
end;

procedure LoadList(const Node: IXmlNode);
var
  d, j: Integer;
  a: Pointer;

  procedure _process(const Node: IXmlNode);
  var
    s, s1, s2: string;
    i, id: Integer;

    procedure _d(const s: string; out s1, s2, s3: string);
    begin
      StrDivide(s, s1, s2, '/');
      StrDivide(s2, s2, s3, '/');
    end;

  begin
    for i := 0 to Node.ChildNodes.Count - 1 do
      if Node.ChildNodes[i].NodeName = 'group' then _process(Node.ChildNodes[i])
      else
        begin
          if Node.ChildNodes[i].NodeName <> 'item' then Continue;
          s := GetAttr(Node.ChildNodes[i], 'val');
          if not (d in [22, 30]) then
            id := GetNumAttr(Node.ChildNodes[i], 'id');
          case d of
            0..11: begin
              LexicArray(a^)[id] := TLexicName.Create;
              LexicArray(a^)[id].Gen := GetNumAttr(Node.ChildNodes[i], 'gen');
              LexicArray(a^)[id].Num := id;
              _d(s, LexicArray(a^)[id].Name[1], LexicArray(a^)[id].Name[2], LexicArray(a^)[id].Name[3]);
              _d(GetAttr(Node.ChildNodes[i], 'lex'), s, s1, s2);
              LexicArray(a^)[id].Template[1] := s.ToInteger;
              LexicArray(a^)[id].Template[2] := s1.ToInteger;
              LexicArray(a^)[id].Template[3] := s2.ToInteger;
            end;
            22: begin
              with GroupsList[j] do
                begin
                  Id := j;
                  Title := GetAttr(Node.ChildNodes[i], 'title');
                  Val := s;
                  Pre := GetAttr(Node.ChildNodes[i], 'pre');
                  PrayType := GetNumAttr(Node.ChildNodes[i], 'type');
                end;
              GroupDic.AddOrSetValue(s, GroupsList[j]);
              Inc(j);
            end;
            23: with SanctLexList[GetNumAttr(Node.ChildNodes[i], 'id')] do
              begin
                FTitle := GetAttr(Node.ChildNodes[i], 'title');
                FCall := GetAttr(Node.ChildNodes[i], 'call');
                FMult := GetAttr(Node.ChildNodes[i], 'mult');
                FFull := GetAttr(Node.ChildNodes[i], 'full');
                FAdd := GetAttr(Node.ChildNodes[i], 'add');
                FExt := GetAttr(Node.ChildNodes[i], 'ext');
                FSort := GetNumAttr(Node.ChildNodes[i], 'sort');
              end;
            24: begin
              PripevsList[id].Pad := GetNumAttr(Node.ChildNodes[i], 'pad');
              if s.IndexOf('/') < 0 then
                PripevsList[id].Text[1] := s
              else _d(s, PripevsList[id].Text[1], PripevsList[id].Text[2], PripevsList[id].Text[3]);
            end;
            25: begin
              UnpackTemplate(s, LexicTemplates[id]);
              LexicTemplates[id].Group := GetNumAttr(Node.ChildNodes[i], 'group');
            end;
            30: begin
              ReplList[j].Id := GetAttr(Node.ChildNodes[i], 'id');
              ReplList[j].Val := s;
              ReplList[j].Nomin := GetNumAttr(Node.ChildNodes[i], 'nomin');
              Inc(j);
            end;
            else TStringDynArray(a^)[id] := s;
          end;
        end;
  end;

begin
  d := IndexStr(GetAttr(Node, 'id'), ListNames);
  j := GetNumAttr(Node, 'count') + 1;
  a := ListArrays[d];
  case d of
    0..11: SetLength(LexicArray(a^), j);
    22: SetLength(GroupsList, j);
    23: SetLength(SanctLexList, j);
    24: SetLength(PripevsList, j);
    25: SetLength(LexicTemplates, j);
    30: SetLength(ReplList, j);
    else SetLength(TStringDynArray(a^), j);
  end;
  j := 1;
  _process(Node);
end;

function GetLexicArray(I: Integer): LexicArray; overload;
begin
  Result := LexicArray(ListArrays[I]^);
end;

function GetLexicArray(Ch: Char): LexicArray; overload;
begin
  Result := LexicArray(ListArrays[_LexicSym.IndexOf(Ch)]^);
end;

end.
