unit EccMemos;

interface

uses
  Generics.Collections, Classes, EccPrays, XmlIntf;

type
  TTitleGenerator = class(TPersistent)
  public
    FTitle, FCall, FFull: string;
    procedure Assign(const Obj: TTitleGenerator);
    function GetStr(Index: Integer): string; virtual;
    property Title: string index 1 read GetStr write FTitle;
    property Call: string index 2 read GetStr write FCall;
    property Full: string index 3 read GetStr write FFull;
  end;

  TPerson = class(TTitleGenerator)
  public
    Id, Nomin, Name: Integer;
    IsSanct: Boolean;
    FAdd, FKrat, FExt, FDop: string;
    Bio, Pic: string;
    Date: TDateTime;
    Comm: string;
    constructor Load(const Node: IXmlNode);
    procedure Assign(const Source: TPerson);
    function GetStr(Index: Integer): string; override;
    function IsAugust: Boolean;
    procedure Store(const Book: string);
    property Info: string index 4 read GetStr;
    property Info2: string index 5 read GetStr;
  end;
  APersons = array of TPerson;

  TGlory = class(TTitleGenerator)
  public
    Id, MiOId, Area, FExt: string;
    TypeNum, Grade, Stared: Integer;
    FSancts: TList<TPerson>;
    constructor Create;
    constructor Load(const Node: IXmlNode);
    destructor Destroy; override;
    procedure Assign(const Source: TGlory);
    function GetStr(Index: Integer): string; override;
    function GetSanct(Index: Integer): TPerson;
    function IsSanctInList(Id: Integer): Boolean;
    function IsSanct: Boolean;
    function GetCount: Integer;
    property Sancts[index: Integer]: TPerson read GetSanct;
    property Count: Integer read GetCount;
  end;

  TMonthDay = packed record
    M, D: SmallInt;
  end;

  TMemory = class
  public
    FGlory, FTitle, Area: string;
    Holi, Relic: Integer;
    Rank: Integer;  {-1 -  !}
    Exact: Integer; { -   ,
                      , }
    ExecNode: IXmlNode;
    Source: Integer; {0: , 1: , 2: , 3: }
    Date: TMonthDay;
    Default: Boolean;
    Services: array of TService;
    constructor Load(const Node, DateNode: IXmlNode);
    function GetTitle: string;
    function GetGlory: TGlory;
    procedure SetGlory(const Value: TGlory);
    property Title: string read GetTitle write FTitle;
    property Glory: TGlory read GetGlory write SetGlory;
  end;

  TMemExemplar = class
    [weak] Memory: TMemory;
    Rank: Integer;
    Active: Boolean;
    ItemIndex: Integer;
  end;

  TMemoriesList = TList<TMemory>;

  TSanctLex = record
    FTitle, FCall, FFull, FAdd, FExt, FMult: string;
    FSort: Integer;
    function GetStr(Index, Num: Integer): string;
  end;

  TDateRec = record
    Rank: Integer;
    MemList: TList<TMemExemplar>;
  end;

  TGloryDic = TDictionary<string, TGlory>;
  TPersonDic = TDictionary<Integer, TPerson>;

var
  MineaList, TriodList, OctList, OtherList: TMemoriesList;
  DatesDic: TDictionary<TDateTime, TDateRec>;
  SanctLexList: array of TSanctLex;
  GloryDic: TGloryDic;
  PersonsDic, SanctsDic: TPersonDic;
  PravArhierey: TPerson;

procedure ResetServMap;
function GetServElm(S: string; Zero: Boolean = False): TPrayList;
function GetPrayParam(PrayRef: string; const Field: string; Rank: Integer): string;
function GetServParam(ServRef: string; const Field: string): string;
function GetGloryParam(Id: string; const Field: string): string;
procedure CreateOctoih;

const
  _PredvarGlories: array[0..7] of string =
    ('pashal', 'troica', 'gospod', 'voskr', 'krestvskr',
    'bogorod', 'krestbgrd', 'krest');
  ShortDWs: array[1..7] of string =
    ('nd', 'pn', 'vt', 'sr', 'ct', 'pt', 'sb');

function SanctsListDecode(const SL: TList<TPerson>; Index: Integer; WriteI: Boolean = False; Count: Integer = 0): string;

implementation

uses
  SysUtils,
  EccDefs, EccProcs, EccCSlav, EccXml, EccScripts, EccLists;

function SanctsListDecode(const SL: TList<TPerson>; Index: Integer; WriteI: Boolean = False; Count: Integer = 0): string;
var
  i, c: Integer;
  s, bn, bn0, n, an, an0, nc: string;

  function _applyI(const s: string): string;
  var
    i: Integer;
  begin
    Result := s;
    SetLength(Result, Length(Result) - 2);
    if WriteI and (SL.Count > 1) then
      begin
        i := Result.LastIndexOf(';');
        Result := Result.Substring(0, i) + ' 3' + Result.Substring(i + 1, MaxLongInt);
      end;
  end;
  function _insertCount(s: string): string;
  var
    x: Integer;
  begin
    Result := s;
    if c = 1 then Exit;
    repeat
      x := s.LastIndexOfAny(['R', 'G', 'T', 'L', 'P']);
      if x = -1 then Exit;
      SetLength(s, x);
      if Result.Chars[x + 1] <> ':' then
        Result := Result.Insert(Result.IndexOfAny([']', #32], x), ':' + IntToStr(c));
    until False;
  end;
  procedure _assembly;
  begin
    if bn0 <> '' then
      begin
        if c > 3 then c := 3;
        Result := Result + _insertCount(bn0).Trim + ' ' +
          nc.Substring(0, Length(nc) - 2) + ' [L:' +
          _insertCount(an0).Trim + '; ';
      end;
    c := 1;
  end;

begin
  if Count = 0 then Count := SL.Count;
  if Count = 0 then Exit;
  if Count = 1 then Exit(SL[0].GetStr(Index));
  Result := '';
  bn0 := '';
  c := 1;
  for i := 0 to Count - 1 do
    begin
      s := SL[i].GetStr(Index);
      if Pos('N', s) = 0 then
        begin
          _assembly;
          Result := Result + s + '; ';
          bn0 := '';
          Continue;
        end;
      StrDivide(s, bn, an, 'N');
      n := 'N' + an.Substring(0, an.IndexOfAny([']', #32]));
      an := an.Substring(Length(n) - 1, MaxLongInt);
      if bn0 = '' then
        begin
          bn0 := bn;
          an0 := an;
          nc := n + ']; ';
          Continue;
        end;
      if (bn = bn0) and (an = an0) then
        begin
          Inc(c);
          nc := nc + '[L:' + n + ']; ';
          if i = Count - 1 then _assembly;
        end
      else
        begin
          _assembly;
          if i = Count - 1 then
            Result := Result + s + '; '
          else
            begin
              nc := n + ']; ';
              bn0 := bn;
              an0 := an;
            end;
        end;
    end;
  Result := _applyI(Result.Replace('] [L:', #32).Replace('[L:]', '').Replace(' ]', ']')).Replace(';', ',');
end;

procedure TMemory.SetGlory(const Value: TGlory);
begin
  FGlory := Value.Id;
end;

function TSanctLex.GetStr(Index, Num: Integer): string;
begin
  case Index of
    1: Result := FTitle;
    2: if not ((Num = 0) and (FMult <> '')) then
         if FCall <> '' then Result := FCall else Result := FTitle
      else
        Result := FMult;
    3: if FFull <> '' then Result := FFull else Result := FTitle;
    4: Result := FAdd;
    5: Result := FExt;
  end;
  case Num of
    1: Result := Result.Replace(':1', '');
    2: Result := Result.Replace(':1', ':2');
    else Result := Result.Replace(':1', ':3');
  end;
end;

{ TPerson }

procedure TPerson.Assign(const Source: TPerson);
begin
  inherited Assign(Source);
  Id := Source.Id;
  IsSanct := Source.IsSanct;
  FAdd := Source.FAdd;
  FExt := Source.FExt;
  FKrat := Source.FKrat;
  FDop := Source.FDop;
  Nomin := Source.Nomin;
  Name := Source.Name;
  Self.Date := Source.Date;
  Bio := Source.Bio;
  Pic := Source.Pic;
  Comm := Source.Comm;
end;

function TPerson.GetStr(Index: Integer): string;
var
  m: string;
  function _nnn(nom: Integer): string;
  begin
    case nom of
      14: Result := 'G27] 3 [L:G14';
      23: Result := 'W4 G23';
      25, 26: Result := 'G24], [L:G26';
      else Result := 'G' + IntToStr(nom);
    end;
  end;
begin
  Result := inherited;
  if Result = '' then
    if IsSanct then
      case Index of
        1: begin
          Result := SanctLexList[Nomin].GetStr(1, 1);
          Insert(' N' + IntToStr(Name), Result, Length(Result));
          m := SanctLexList[Nomin].GetStr(4, 1);
          if m <> '' then Insert(#32 + UnScob(m), Result, Length(Result));
          if FKrat <> '' then Insert(#32 + UnScob(FKrat), Result, Length(Result));
          if FAdd <> '' then Insert(#32 + UnScob(FAdd), Result, Length(Result));
          m := SanctLexList[Nomin].GetStr(5, 1);
          if m <> '' then Result := Result + ', ' + m;
          if (FKrat = '') and (FExt <> '') then Result := Result + ', ' + FExt;
        end;
        2: begin
          Result := SanctLexList[Nomin].GetStr(2, 1);
          Insert(' N' + IntToStr(Name), Result, Length(Result));
        end;
        3: begin
          if (FTitle <> '') and (FExt = '') then
            begin
              Result := GetStr(1);
              if Result <> '' then Exit;
            end;
          Result := SanctLexList[Nomin].GetStr(3, 1);
          Insert(' N' + IntToStr(Name), Result, Length(Result));
          m := SanctLexList[Nomin].GetStr(4, 1);
          if m <> '' then Insert(#32 + UnScob(m), Result, Length(Result));
          if FAdd <> '' then Insert(#32 + UnScob(FAdd), Result, Length(Result));
          m := SanctLexList[Nomin].GetStr(5, 1);
          if m <> '' then Result := Result + ', ' + m;
          if FExt <> '' then Result := Result + ', ' + FExt;
          if FDop <> '' then Result := Result + ', ' + FDop;
        end;
        4: begin
          Result := '[L:N' + IntToStr(Name) + ']';
          m := '';
          if Pos('T', FKrat) > 0 then
            m := Copy(FKrat, Pos('T', FKrat), MaxInt)
          else if Pos('T', FExt) > 0 then
            m := Copy(FExt, Pos('T', FExt), MaxInt);
          if m <> '' then
            begin
              m := m + #32;
              if Pos(#32, m) < Pos(']', m) then m := Copy(m, 1, Pos(#32, m) - 1)
              else m := Copy(m, 1, PosExR(']', m) - 1);
              Insert(#32 + m, Result, Length(Result));
            end;
          if FAdd <> '' then Insert(#32 + UnScob(FAdd), Result, Length(Result));
          Result := Translit(ApplyLexic(Result + ', ' + '[L:' + _nnn(Nomin) + ']', 1));
          if Comm <> '' then Result := Result + ' (' + Comm + ')';
        end;
        5: begin
          Result := '[L:' + _nnn(Nomin) + '], [L:N' + IntToStr(Name) + ']';
          m := '';
          if Pos('T', FKrat) > 0 then
            m := Copy(FKrat, Pos('T', FKrat), MaxInt)
          else if Pos('T', FExt) > 0 then
            m := Copy(FExt, Pos('T', FExt), MaxInt);
          if m <> '' then
            begin
              m := m + #32;
              if Pos(#32, m) < Pos(']', m) then m := Copy(m, 1, Pos(#32, m) - 1)
              else m := Copy(m, 1, PosExR(']', m) - 1);
              Insert(#32 + m, Result, Length(Result));
            end;
          if FAdd <> '' then Insert(#32 + UnScob(FAdd), Result, Length(Result));
          Result := Translit(ApplyLexic(Result, 1));
          if Comm <> '' then Result := Result + ' (' + Comm + ')';
        end;
      end
    else
      case Index of
        1..3: begin
          if (Index = 2) and IsAugust then
            begin
              if Nomin = 27 then
                Result := '[L:G40 R' + IntToStr(Nomin) + ' L28]'
              else
                Result := '[L:G39 R' + IntToStr(Nomin) + ' L11]';
            end
          else
            begin
              Result := FAdd;
              if Result = '' then Result := '[L:]';
            end;
          if (Self = PravArhierey) and (Pos('L11]', Result) = 0) then
            if Pos('L10]', Result) > 0 then Result := Result.Replace('L10]', 'L10 L11]')
            else if Pos('L15]', Result) > 0 then Result := Result.Replace('L15]', 'L15 L11]');
          if (Index <> 1) and not IsAugust then
            Insert(#32 + UnScob(FKrat), Result, Length(Result));
          if ((Index = 2) and not IsAugust) or ((Index = 1) and IsAugust) then
            if FDop <> '' then
              Insert(#32 + UnScob(FDop), Result, Length(Result))
            else
              Insert(' R' + IntToStr(Nomin), Result, Length(Result));
          Insert(' N' + IntToStr(Name), Result, Length(Result));
          if IsAugust then
            Insert(#32 + UnScob(FKrat), Result, Length(Result));
          if (Index = 1) and not IsAugust then
            Result := Result + ', ' + FKrat;
          if (Index = 1) and not IsAugust then
            if FDop <> '' then
              Insert(#32 + UnScob(FDop), Result, Length(Result))
            else
              Insert(' R' + Nomin.ToString, Result, Length(Result));
          if Index = 3 then
            if FDop <> '' then
              Result := Result + ', ' + FDop
            else
              Result := Result + ', [L:R' + Nomin.ToString + ']';
          if Index <> 2 then
            if FExt <> '' then Result := Result + #32 + FExt;
          Result := Result.Replace('] [L:', #32);
        end;
        4: begin
          Result := '[L:N' + Name.ToString + ']';
          if IsAugust then
            Insert(#32 + UnScob(FKrat), Result, Length(Result));
          if Comm <> '' then Result := Result + ' (*)';
          if FDop <> '' then Result := Result + ', ' + FDop
          else
            Result := Result + ', [L:R' + IntToStr(Nomin) + ']';
          if FExt <> '' then Result := Result + #32 + FExt;
          Result := Translit(ApplyLexic(Result, 1)).Replace('*', Comm);
        end;
        5: begin
          if FDop <> '' then Result := FDop
          else
            Result := '[L:R' + IntToStr(Nomin) + ']';
          Result := Result + ', [L:N' + Name.ToString + ']';
          if IsAugust then
            Insert(#32 + UnScob(FKrat), Result, Length(Result));
          if Comm <> '' then Result := Result + ' (*)';
          if FExt <> '' then Result := Result + #32 + FExt;
          Result := Translit(ApplyLexic(Result, 1)).Replace('*', Comm);
        end;
      end;
end;

function TPerson.IsAugust: Boolean;
begin
  Result := Nomin in [26..28];
end;

{ TGlory }

procedure TGlory.Assign(const Source: TGlory);
var
  i: TPerson;
begin
  inherited Assign(Source);
  Id := Source.Id;
  FExt := Source.FExt;
  MiOId := Source.MiOId;
  Area := Source.Area;
  TypeNum := Source.TypeNum;
  Grade := Source.Grade;
  Stared := Source.Stared;
  FSancts.Clear;
  for i in Source.FSancts do FSancts.Add(i);
end;

constructor TGlory.Create;
begin
  inherited;
  FSancts := TList<TPerson>.Create;
end;

destructor TGlory.Destroy;
begin
  FSancts.Free;
  inherited;
end;

function TGlory.GetCount: Integer;
begin
  try Result := Stared except Result := 0 end;
end;

function TGlory.GetSanct(Index: Integer): TPerson;
begin
  Result := FSancts[Index];
end;

function TGlory.GetStr(Index: Integer): string;
begin
  Result := inherited;
  if (Index = 3) and (FFull = '') and (FTitle <> '') then Result := GetStr(1);
  if Result = '' then
    if (Index = 3) and (FSancts.Count > 0) then
      Result := SanctsListDecode(FSancts, Index, True)
    else if Stared in [1..3] then
      Result := SanctsListDecode(FSancts, Index, True, Stared)
    else
      Result := SanctLexList[Grade].GetStr(Index, 0);
  if Index <> 2 then if FExt <> '' then Result := Result + ', ' + FExt;
end;

function TGlory.IsSanct: Boolean;
begin
  Result := not (Grade in [37..38]);
end;

function TGlory.IsSanctInList(Id: Integer): Boolean;
var
  i: TPerson;
begin
  Result := False;
  for i in FSancts do
    if i.Id = Id then Result := True;
end;

{ TTitleGenerator }

procedure TTitleGenerator.Assign(const Obj: TTitleGenerator);
begin
  FTitle := Obj.FTitle;
  FCall := Obj.FCall;
  FFull := Obj.FFull;
end;

function TTitleGenerator.GetStr(Index: Integer): string;
begin
  case Index of
    1: Result := FTitle;
    2: Result := FCall;
    3: Result := FFull;
    else Result := '';
  end;
end;

{ TMemory }

function TMemory.GetGlory: TGlory;
begin
  try
    Result := GloryDic[FGlory];
  except
    Result := nil;
  end;
end;

function TMemory.GetTitle: string;
begin
  if FTitle <> '' then Result := FTitle
  else
    begin
      Result := ApplyLexic('[L:H' + IntToStr(Holi) + ']', 1) + #32;
      if Relic <= 0 then
        Result := Result + ApplyLexic(Glory.Title, 2)
      else
        Result := Result + RelicsList[Relic].Replace('[*]', ApplyLexic(Glory.Title, 2));
    end;
end;

function GetServElm(S: string; Zero: Boolean = False): TPrayList;
//        
var
  serv: Integer;
begin
  if not Zero then StrNum2Div(S, serv, S) else serv := 0;
  try
    Result := ServMap[serv].Prays[GroupDic[S].Id];
  except
    Result := nil;
  end;
end;

function GetPrayParam(PrayRef: string; const Field: string; Rank: Integer): string;
var
  id: Integer;
  pl: TPrayList;
  pr: TPrayRec;
begin
  case PrayRef.Chars[0] of
    '^': begin
      for id := PrayRef.Length - 2 downto 2 do
        if PrayRef.Chars[id] = '.' then Break;
      Result := PrayRef.Substring(id + 1);
      PrayRef := PrayRef.Substring(0, id);
      id := Result.ToInteger;
      pl := GetServElm(PrayRef);
      if id < 0 then id := pl.Count + id + 1;
      if Field = 'index' then
        Exit(id.ToString); //   
      if (id < 1) or (id > pl.Count) then Exit('');
      pr := pl[id - 1];
    end;
    '@': begin
      for id := PrayRef.Length - 2 downto 2 do
        if PrayRef.Chars[id] = '.' then Break;
      Result := PrayRef.Substring(id + 1);
      PrayRef := PrayRef.Substring(0, id);
      id := Result.ToInteger;
      pl := LoadFromExternService(PrayRef.Substring(1), Rank);
      if id < 0 then id := pl.Count + id + 1;
      if Field = 'index' then
        Exit(id.ToString); //   
      if (id < 1) or (id > pl.Count) then Exit('');
      pr := pl[id - 1];
    end;
    else FindOrLoadPray(PrayRef).ToRec(pr);
  end;
  if Field = 'id' then Exit(pr.Id);
  if Field = 'text' then Exit(pr.Text);
  if Field = 'short' then Exit(TruncPray(pr.Text));
  if Field = 'alias' then Exit(pr.Alias);
  if Field = 'subv' then Exit(pr.SubVoice.ToString);
  if Field = 'voice' then Exit(pr.Voice.ToString);
  if Field = 'type' then Exit(pr.PrayType.ToString);
  if Field = 'role' then Exit(pr.RolesStr);
  if Field = 'style' then Exit(pr.Style.ToString);
  if Field = 'glory' then Exit(pr.GloryId);
  if Field = 'holi' then Exit(pr.Holi.ToString);
  if Field = 'relic' then Exit(pr.Relic.ToString);
  if Field = 'iter' then Exit(pr.Iter.ToString); //  
  if Field = 'title' then Exit(pr.Title);
  if Field = 'author' then Exit(pr.Author.ToString);
  if Field = 'count' then           //  
    if pr.Text.IsEmpty then Result := '0'
    else Result := (pr.Text.CountChar(#10) + 1).ToString;
end;

function GetServParam(ServRef: string; const Field: string): string;
var
  id: Integer;
begin
  if ServRef.Chars[0] = '^' then
    begin
      id := string(ServRef.Chars[1]).ToInteger;
      if Field = 'id' then Exit(ServMap[id].Id);
      if Field = 'title' then Exit(ServMap[id].Title);
      if Field = 'glory' then Exit(ServMap[id].GloryId);
      if Field = 'holi' then Exit(ServMap[id].Holi.ToString);
      if Field = 'relic' then Exit(ServMap[id].Relic.ToString);
      if Field = 'rank' then Exit(ServMap[id].Rank.ToString);
      if Field = 'exact' then Exit(ServMap[id].Exact.ToString);
    end
  else
    Result := GetAttr(FindService(ServRef), Field);
end;

function GetGloryParam(Id: string; const Field: string): string;
var
  pl: TList<TPerson>;
  i: Integer;
begin
  if Field = 'title' then Exit(GloryDic[id].Title);
  if Field = 'full' then Exit(GloryDic[id].Full);
  if Field = 'mioid' then Exit(GloryDic[id].MiOId);
  if Field = 'type' then Exit(GloryDic[id].TypeNum.ToString);
  if Field = 'grade' then Exit(GloryDic[id].Grade.ToString);
  if Field = 'star' then Exit(GloryDic[id].Stared.ToString);
  pl := GloryDic[id].FSancts;
  if Field = 'count' then Exit(pl.Count.ToString);
  if Field = 'name' then
    if GloryDic[id].Stared = 1 then
      Exit(pl[0].Name.ToString)
    else Exit('');
  if Field = 'sancts' then
    for i := 0 to pl.Count - 1 do
      Result := Result + pl[i].Id.ToString + ', ';
  Result := Result.Substring(0, Result.Length - 2);
end;

procedure ResetServMap;
var
  i: Integer;
  pl: TPrayList;
begin
  for i := 0 to High(ServMap) do
    begin
      if i < 9 then ServMap[i].Id := '';
      for pl in ServMap[i].Prays do pl.Free;
      SetLength(ServMap[i].Prays, 0);
    end;
  SetLength(ServMap[0].Prays, 280);
  for i := 0 to High(ServMap[0].Prays) do
    ServMap[0].Prays[i] := TPrayList.Create;
end;

constructor TGlory.Load(const Node: IXmlNode);
var
  s: string;
begin
  FSancts := TList<TPerson>.Create;
  Id := GetAttr(Node, 'id');
  MiOId := GetAttr(Node, 'mioid');
  FCall := GetAttr(Node, 'call');
  FTitle := GetAttr(Node, 'title');
  FFull := GetAttr(Node, 'full');
  TypeNum := GetNumAttr(Node, 'type');
  Grade := GetNumAttr(Node, 'grade');
  Area := GetAttr(Node, 'area');
  Stared := GetNumAttr(Node, 'star');
  s := Trim(GetAttr(Node, 'sancts'));
  if s <> '' then
    begin
      s := s + ',';
      while s <> '' do
        begin
          FSancts.Add(SanctsDic[ToInt(Copy(s, 1, Pos(',', s) - 1))]);
          System.Delete(s, 1, Pos(',', s));
          s := Trim(s);
        end;
    end;
  GloryDic.Add(Id, Self);
end;

constructor TPerson.Load(const Node: IXmlNode);
var
  i: Integer;
  n: IXmlNode;
begin
  IsSanct := (Node.NodeName = 'sanct');
  Id := GetNumAttr(Node, 'id');
  Name := GetNumAttr(Node, 'name');
  FCall := GetAttr(Node, 'call');
  FTitle := GetAttr(Node, 'title');
  FFull := GetAttr(Node, 'full');
  FKrat := GetAttr(Node, 'krat');
  FExt := GetAttr(Node, 'ext');
  Comm := GetAttr(Node, 'comm');
  for i := 0 to Node.ChildNodes.Count - 1 do
    begin
      n := Node.ChildNodes[i];
      if n.NodeName = 'bio' then Bio := n.Text;
      if n.NodeName = 'pic' then Pic := GetAttr(n, 'url');
    end;
  if IsSanct then
    begin
      FAdd := GetAttr(Node, 'add');
      FDop := GetAttr(Node, 'dop');
      Nomin := GetNumAttr(Node, 'grade');
      SanctsDic.Add(Id, Self);
    end
  else
    begin
      FAdd := GetAttr(Node, 'pref');
      FDop := GetAttr(Node, 'nomin');
      Nomin := GetNumAttr(Node, 'rank');
      PersonsDic.Add(Id, Self);
    end;
end;

constructor TMemory.Load(const Node, DateNode: IXmlNode);
var
  i: Integer;
  s, s1: string;
begin
  s := GetAttr(Node, 'glory');
  StrDivide(s, s, s1, '.');
  FGlory := s;
  StrDivide(s1, s1, s, '.');
  if not s1.IsEmpty then Holi := s1.ToInteger else Holi := 0;
  if not s.IsEmpty then Relic := s.ToInteger else Relic := 0;
  Area := GetAttr(Node, 'area');
  Default := (GetNumAttr(Node, 'default') = 1);
  if DateNode = nil then Source := 4
  else
    begin
      Date.M := GetNumAttr(DateNode, 'mon');
      Date.D := GetNumAttr(DateNode, 'day');
      if Date.M = 0 then Source := 3 else Source := 2;
    end;
  Exact := GetNumAttr(Node, 'exact');
  FTitle := GetAttr(Node, 'title');
  Rank := GetNumAttr(Node, 'rank');
  for i := 0 to Node.ChildNodes.Count - 1 do
    if Node.ChildNodes[i].NodeName = 'exec' then ExecNode := Node.ChildNodes[i]
    else
      begin
        SetLength(Services, Length(Services) + 1);
        Services[High(Services)].Id := GetAttr(Node.ChildNodes[i], 'ref');
        Services[High(Services)].Rank := GetNumAttr(Node.ChildNodes[i], 'rank');
      end;
  case Source of
    2: MineaList.Add(Self);
    3: TriodList.Add(Self);
    else OtherList.Add(Self);
  end;
end;

procedure TPerson.Store(const Book: string);
var
  n, m: IXmlNode;
begin
  XmlCode.LoadFromFile(GetBookPath(Book));
  if IsSanct then n := FindOrCreateNode(XmlCode.DocumentElement, 'sanct', IntToStr(Id))
  else n := FindOrCreateNode(XmlCode.DocumentElement, 'pers', IntToStr(Id));
  AssignAttr(n, 'title', FTitle);
  AssignAttr(n, 'call', FCall);
  AssignAttr(n, 'krat', FKrat);
  AssignAttr(n, 'ext', FExt);
  if IsSanct then
    begin
      AssignAttr(n, 'add', FAdd);
      AssignAttr(n, 'dop', FDop);
      AssignAttr(n, 'grade', IntToStr(Nomin))
    end
  else
    begin
      AssignAttr(n, 'pref', FAdd);
      AssignAttr(n, 'nomin', FDop);
      AssignAttr(n, 'rank', IntToStr(Nomin));
    end;
  AssignAttr(n, 'full', FFull);
  AssignAttr(n, 'name', IntToStr(Name));
  AssignAttr(n, 'date', DateToStr(Date));
  n.ChildNodes.Clear;
  if Bio <> '' then
    begin
      m := n.AddChild('bio');
      m.NodeValue := Bio;
    end;
  if Pic <> '' then
    begin
      m := n.AddChild('pic');
      m.Attributes['url'] := Pic;
    end;
  AssignAttr(n, 'comm', Comm);
  XmlCode.SaveToFile(GetBookPath(Book));
end;

procedure CreateOctoih;
var
  i, j: Integer;
  s: string;
  mem: TMemory;
begin
  for i := 1 to 8 do
    for j := 1 to 7 do
      begin
        mem := TMemory.Create;
        mem.Holi := j;
        mem.Source := 1;
        mem.Default := True;
        mem.Area := '';
        mem.Exact := 9;
        mem.FGlory := 'vsesv';
        mem.Title := HolidaysList[j].Name[1] + #32 + CSNum(i) + ApplyLexic(' [L:L42]', 2);
        SetLength(mem.Services, 1);
        mem.Services[0].Id := 'oc_' + IntToStr(i) + '.' + ShortDWs[j];
        mem.Rank := 6 * Ord(j = 1);
        mem.Services[0].Rank := mem.Rank;
        OctList.Add(mem);
      end;
end;

initialization
  GloryDic := TGloryDic.Create;
  PersonsDic := TPersonDic.Create;
  SanctsDic := TPersonDic.Create;
  MineaList := TMemoriesList.Create;
  TriodList := TMemoriesList.Create;
  OtherList := TMemoriesList.Create;
  DatesDic := TDictionary<TDateTime, TDateRec>.Create;
finalization
  GloryDic.Free;
  PersonsDic.Free;
  SanctsDic.Free;
  MineaList.Free;
  TriodList.Free;
  OtherList.Free;
  DatesDic.Free;
end.
