unit EccPrays;

interface

uses
  Types, Classes, Generics.Collections, XmlIntf;

type
  TRoleRec = packed record
    Role, Subr, Count: ShortInt;
  end;

  TRolesList = array of TRoleRec;

  TPrayRec = record
    Id, Title, Text, Alias, Subscr, GloryId: string;
    Mode, Voice, SubVoice, Style, PrayType, Holi, Relic,
    Author, Iter, Duration: Integer;
    Roles: TRolesList;
    Items: TList<TList<TPrayRec>>;
    procedure AdjustFromNode(const Node: IXmlNode; const Params: TStringDynArray; const Book: string = '');
    procedure SetRoles(S: string);
    function RolesCount: Integer;
    function RolesStr: string;
  end;

  TPray = class(TPersistent)
  public
    Id, Title, Text, Alias, GloryId: string;
    Mode, PrayType, Voice, SubVoice, Author: Integer;
    function Load(Id: string): Boolean;
    function TranslateComp(const Id: string): Boolean;
    procedure ToRec(var P: TPrayRec);
    procedure FromRec(const P: TPrayRec);
    procedure AdjustFromNode(const Node: IXmlNode; const Book: string = '');
    procedure Store;
  end;

  TPrayList = TList<TPrayRec>;
  TPrayDic = TDictionary<string, TPray>;

  TService = record
    Id: string;
    Rank, SortIdx, Exact, Holi, Relic: Integer;
    IsMov: Boolean;
    GloryId: string;
    FMiOId: string;
    Title: string;
    Prays: array of TPrayList;
    function MiOId: string;
  end;

  TServiceList = TList<TService>;

var
  PrayDic, ActDic, ImgDic: TPrayDic;
  PrayList: TPrayList;
  ServMap: array[0..9] of TService;

function FindOrLoadPray(Id: string): TPray;
function LoadFromExternService(Id: string; Rank: Integer): TPrayList;
function FindGroupInExternService(Id: string; Rank: Integer): IXmlNode;
procedure LoadServices;
function FindService(Id: string): IXmlNode;
procedure _Copy_Instr(const Node: IXmlNode; Params: TStringDynArray);
procedure _Addall_Instr(Id: string; const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList);
procedure _Del_Instr(const Node: IXmlNode; var Params: TStringDynArray);
procedure _Text_Instr(const Node: IXmlNode; Params: TStringDynArray; const List: TPrayList; IsTit: Boolean = False);
procedure AddPray(Id: string; const Node: IXmlNode; const Params: TStringDynArray; const List: TPrayList; Extr: Boolean = False);
procedure AddAct(const Id: string; const Node: IXmlNode; Params: TStringDynArray; const List: TPrayList);
procedure AddImg(const Id: string; const Node: IXmlNode; const List: TPrayList);
function UnpackRoles(S: string): TRolesList;

implementation

uses
  SysUtils,
  EccDefs, EccXml, EccProcs, EccMemos, EccScripts, EccVars,
  EccCSlav, EccLists;

function InRank(const Node: IXmlNode; Rank: Integer): Boolean;
begin
  Result := (GetNumAttr(Node, 'min') <= Rank) and ((GetNumAttr(Node, 'max') >= Rank) or (GetNumAttr(Node, 'max') = -1));
end;

function FindExternServiceBlock(Id: string; Rank: Integer): IXmlNode;
//       
var
  sn: IXmlNode;
  i: Integer;
  s: string;
begin
  StrDivide(Id, Id, s, '.', True);
  sn := FindService(Id);
  Result := nil;
  for i := 0 to sn.ChildNodes.Count - 1 do
    if (sn.ChildNodes[i].NodeName = 'block') and (GetAttr(sn.ChildNodes[i], 'id') = s)
    and InRank(sn.ChildNodes[i], Rank) then Exit(sn.ChildNodes[i]);
end;

function FindService(Id: string): IXmlNode;
//      
var
  s: string;
  i: Integer;
begin
  StrDivide(Id, s, Id, '.');
  XmlCode.LoadFromFile(GetBookPath(s));
  for i := 0 to XmlCode.DocumentElement.ChildNodes.Count - 1 do
    if (XmlCode.DocumentElement.ChildNodes[i].NodeName = 'serv') and (GetAttr(XmlCode.DocumentElement.ChildNodes[i], 'id') = Id) then
      Exit(XmlCode.DocumentElement.ChildNodes[i]);
end;

procedure AddPray(Id: string; const Node: IXmlNode; const Params: TStringDynArray; const List: TPrayList; Extr: Boolean = False);
var
  p: TPrayRec;
  n: Integer;
  s: string;
  Start, Fin, Divi: Integer;
  sl: TStringList;
begin
  if Id[1] <> '^' then
    FindOrLoadPray(Id).ToRec(p)
  else
    begin
      StrNum2Div(Id, n, Id);
      StrDivide(Id, s, Id, '.', True);
      fin := StrToInt(Id);
      start := GroupDic[s].Id;
      if fin > 0 then p := ServMap[n].Prays[start][fin - 1]
      else p := ServMap[n].Prays[start][ServMap[n].Prays[start].Count + fin];
    end;
  if Extr then
    begin
      Start := GetNumAttr(Node, 'start', Params);
      if Start < 1 then Start := 1;
      Fin := GetNumAttr(Node, 'end', Params);
      Divi := GetNumAttr(Node, 'div', Params);
      if Divi <= 0 then
        begin
          if Fin < Start then Fin := Start;
          sl := TStringList.Create;
          sl.Text := p.Text;
          if Fin > sl.Count then Fin := sl.Count;
          p.Text := '';
          for n := Start - 1 to Fin - 1 do
            if p.Text <> '' then p.Text := p.Text + #13#10 + sl[n]
            else p.Text := sl[n];
          sl.Free;
        end
      else
        begin
          sl := TStringList.Create;
          sl.Text := p.Text;
          p.Text := '';
          if Fin < Divi then Fin := Divi;
          for n := Divi to Fin do
            p.Text := p.Text + #32 + Trim(StrElm(sl[Start - 1], n, '/'));
          p.Text := End2Dot(Trim(p.Text));
          if GetNumAttr(Node, 'lin') < 1 then p.Text := FirstCap(p.Text);
          sl.Free;
        end;
      p.Title := '';
    end;
  if Node.HasAttribute('glory') then p.GloryId := GetAttr(Node, 'glory');
  if not ApplyPrayAll(p, Params) then Exit;
  p.AdjustFromNode(Node, Params, '');
  List.Add(p);
end;

procedure SetGlory(var P: TPrayRec; const G: string);
begin
  case p.PrayType of
    10, 11, 14, 30, 31: p.GloryId := 'bogorod';
    16, 39: p.GloryId := 'troica';
    18, 19, 38: p.GloryId := 'muchs';
    else p.GloryId := G;
  end;
end;

procedure LoadServices;
var
  Index, i, j, k, n: Integer;
  min, max: ShortInt;
  s, id, d, g1, g2, g3: string;
  sn, sb, s3, s4: IXmlNode;
  p: TPrayRec;
  _junk: TStringDynArray;
  rank: Integer;

  function _glory: string;
  begin
    if g3 <> '' then Exit(g3);
    if g2 <> '' then Exit(g2);
    if g1 <> '' then Exit(g1);
    Result := ServMap[Index].GloryId;
  end;

  function _inminmax(const x: IXmlNode): Boolean;
  begin
    min := GetNumAttr(x, 'min');
    if x.HasAttribute('max') then max := GetNumAttr(x, 'max') else max := 10;
    if max < min then max := 10;
    Result := rank in [min..max];
  end;

begin
  for Index := 1 to 9 do
    try
      id := ServMap[Index].Id;
      if id = '' then Continue;
      LoadDics(id.Substring(0, id.IndexOf('.')));
      sn := FindService(Id);
      if (ServMap[Index].GloryId = '') or (ServMap[Index].Exact = 9) then
        ServMap[Index].GloryId := GetAttr(sn, 'glory');
      SetLength(ServMap[Index].Prays, Length(GroupsList));
      for i := 0 to High(ServMap[Index].Prays) do
        ServMap[Index].Prays[i] := TPrayList.Create;
      for i := 0 to sn.ChildNodes.Count - 1 do
        begin
          sb := sn.ChildNodes[i];
          if sb.NodeName = 'exec' then
            begin
              Vars.AddOrSetValue('Loading.Index', Index.ToString);
              Vars.AddOrSetValue('Loading.Rank', rank.ToString);
              ProcessChild(sb, _junk, nil);
            end;
          if sb.NodeName = 'var' then
            begin
              if GetAttr(sb, 'mode') = 'set' then
                Vars.AddOrSetValue('ServMap[' + Index.ToString + '].' + GetAttr(sb, 'id'), GetAttr(sb, 'val'));
              Continue;
            end;
          if sb.NodeName <> 'block' then Continue;
          if not _inminmax(sb) then Continue;
          s := GetAttr(sb, 'ref');
          if s <> '' then
            if s.Chars[0] = '@' then sb := FindExternServiceBlock(s.Substring(1), Rank)
            else sb := StructDic[s];
          s := GetAttr(sn.ChildNodes[i], 'id');
          g1 := GetAttr(sb, 'glory');
          for j := 0 to sb.ChildNodes.Count - 1 do
            begin
              s3 := sb.ChildNodes[j];
              if s3.NodeName <> 'group' then Continue;
              if not _inminmax(s3) then Continue;
              Id := s + '.' + GetAttr(s3, 'id');
              try
                n := GroupDic[Id].Id;
                g2 := GetAttr(s3, 'ref');
                if g2 <> '' then
                  for p in ServMap[Index].Prays[GroupDic[g2].Id] do
                    ServMap[Index].Prays[n].Add(p)
                else
                  begin
                    g2 := GetAttr(s3, 'glory');
                    for k := 0 to s3.ChildNodes.Count - 1 do
                      begin
                        s4 := s3.ChildNodes[k];
                        if s4.NodeName <> 'item' then Continue;
                        if not _inminmax(s4) then Continue;
                        g3 := GetAttr(s4, 'glory');
                        try
                          d := GetAttr(s4, 'id');
                          FindOrLoadPray(d).ToRec(p);
                        except
                          {$IFDEF DEBUG}
                          OutMessage('   :'#13#10 + ServMap[Index].Id + '.' + Id + ' : ' + d);
                          {$ENDIF}
                        end;
                        SetGlory(p, _glory);
                        p.Holi := ServMap[Index].Holi;
                        ServMap[Index].Prays[n].Add(p);
                      end;
                  end;
              except
                {$IFDEF DEBUG}
                OutMessage('   :'#13#10 + ServMap[Index].Id + '.' + Id);
                {$ENDIF}
              end;
            end;
        end;
    except
      {$IFDEF DEBUG}
      OutMessage('  : ' + ServMap[Index].Id);
      {$ENDIF}
      raise;
    end;
end;

function LoadFromExternService(Id: string; Rank: Integer): TPrayList;
//        
var
  cnj: IXmlNode;
  k: Integer;
  min, max: ShortInt;
  p: TPrayRec;
  g: string;
begin
  Result := TPrayList.Create;
  cnj := FindGroupInExternService(Id, Rank);
  if Assigned(cnj) then
    begin
      g := GetAttr(cnj, 'glory');
      if g = '' then g := GetAttr(cnj.ParentNode, 'glory');
      if g = '' then g := GetAttr(cnj.ParentNode.ParentNode, 'glory');
      for k := 0 to cnj.ChildNodes.Count - 1 do
        begin
          if cnj.ChildNodes[k].NodeName <> 'item' then Continue;
          min := GetNumAttr(cnj.ChildNodes[k], 'min');
          if cnj.ChildNodes[k].HasAttribute('max') then
            max := GetNumAttr(cnj.ChildNodes[k], 'max')
          else max := -1;
          if max < min then max := 10;
          if not (Rank in [min..max]) then Continue;
          FindOrLoadPray(GetAttr(cnj.ChildNodes[k], 'id')).ToRec(p);
          p.GloryId := GetAttr(cnj.ChildNodes[k], 'glory');
          if p.GloryId = '' then p.GloryId := g;
          Result.Add(p);
        end;
    end;
end;

function FindGroupInExternService(Id: string; Rank: Integer): IXmlNode;
var
  sn, cni, cnj: IXmlNode;
  i, j: Integer;
  s, s1: string;
begin
  i := Id.IndexOf('.', Id.IndexOf('.') + 1);
  sn := FindService(Id.Substring(0, i));
  Id := Id.Substring(i + 1, MaxLongInt);
  StrDivide(Id, s1, Id, '.');
  for i := 0 to sn.ChildNodes.Count - 1 do
    begin
      cni := sn.ChildNodes[i];
      if (cni.NodeName <> 'block') or (GetAttr(cni, 'id') <> s1) then Continue;
      s := GetAttr(cni, 'ref');
      if s <> '' then
        if s.Chars[0] = '@' then cni := FindExternServiceBlock(s.Substring(1), Rank)
        else cni := StructDic[s];
      for j := 0 to cni.ChildNodes.Count - 1 do
        begin
          cnj := cni.ChildNodes[j];
          if (cnj.NodeName = 'group') and (GetAttr(cnj, 'id') = Id) then
            Exit(cnj);
        end;
    end;
  Result := nil;
end;

function LoadFromSinglePray(Id: string): TPrayList;
//      
var
  p: TPrayRec;
begin
  Result := TPrayList.Create;
  FindOrLoadPray(Id).ToRec(p);
  Result.Add(p);
end;

procedure _Addall_Instr(Id: string; const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList);
var
  p: TPrayRec;
  i, n: Integer;
begin
  StrNum2Div(Id, n, Id);
  for i := 0 to ServMap[n].Prays[GroupDic[Id].Id].Count - 1 do
    begin
      p := ServMap[n].Prays[GroupDic[Id].Id][i];
      if not ApplyPrayAll(p, Params) then Exit;
      p.AdjustFromNode(Node, Params, '');
      List.Add(p);
    end;
end;

procedure AddAct(const Id: string; const Node: IXmlNode; Params: TStringDynArray; const List: TPrayList);
var
  p: TPrayRec;
begin
  ActDic[Id].ToRec(p);
  if p.Text = '' then Exit;
  p.Id := Id;
  p.Mode := 1;
{  p.Style := GetNumAttr(Node, 'img');}
  p.AdjustFromNode(Node, Params, '');
  p.Alias := GetAttr(Node, 'obj', Params);
  p.Text := ApplyObj(ApplyPV(ApplyFlags(p.Text), Params), P.Alias);
  List.Add(p);
end;

procedure AddImg(const Id: string; const Node: IXmlNode; const List: TPrayList);
var
  p: TPrayRec;
begin
  ImgDic[Id].ToRec(p);
  if p.Text = '' then Exit;
  p.Id := Id;
  p.Mode := 4;
  p.Text := ApplyFlags(p.Text);
  List.Add(p);
end;

procedure TPray.Store;
var
  n: IXmlNode;
begin
  case Mode of
    0: begin
      n := FindNodeWithId(XmlCode.DocumentElement, 'pray', Id);
      if n = nil then
        n := FindOrCreateNode(XmlCode.DocumentElement, 'pray', Copy(Id, Pos('.', Id) + 1, MaxInt));
      AssignAttr(n, 'type', PrayType);
      if SubVoice > 0 then
        AssignAttr(n, 'voice', Voice.ToString + '.' + SubVoice.ToString)
      else
        AssignAttr(n, 'voice', Voice);
      AssignAttr(n, 'author', Author);
    end;
    1: begin
      n := FindNodeWithId(XmlCode.DocumentElement, 'act', Id);
      if n = nil then
        n := FindOrCreateNode(XmlCode.DocumentElement, 'act', Id);
    end;
  end;
  AssignAttr(n, 'title', Title);
  n.Text := Text;
end;

procedure _Text_Instr(const Node: IXmlNode; Params: TStringDynArray; const List: TPrayList; IsTit: Boolean = False);
var
  p: TPrayRec;
begin
  if IsTit then
    begin
      p.Mode := 3;
      p.Id := GetAttr(Node, 'id', Params);
    end
  else
    begin
      p.Mode := 0;
      p.AdjustFromNode(Node, Params, '');
      p.Iter := 1;
      p.GloryId := '';
    end;
  p.Text := ApplyPV(Node.Text, Params);
  List.Add(p);
end;

procedure TPray.ToRec(var P: TPrayRec);
begin
  P.Id := Id;
  p.Title := Title;
  p.Text := Text;
  p.Alias := Alias;
  p.Author := Author;
  p.Subscr := '';
  p.Mode := Mode;
  SetLength(p.Roles, 0);
  p.Style := 0;
  p.PrayType := PrayType;
  p.Voice := Voice;
  p.SubVoice := SubVoice;
  p.Iter := 1;
  if GloryId <> '' then p.GloryId := GloryId;
end;

procedure TPray.FromRec(const P: TPrayRec);
begin
  Id := P.Id;
  Title := p.Title;
  Text := p.Text;
  Alias := p.Alias;
  Author := p.Author;
  Mode := p.Mode;
  PrayType := p.PrayType;
  Voice := p.Voice;
  SubVoice := p.SubVoice;
  GloryId := p.GloryId;
end;

function TPray.TranslateComp(const Id: string): Boolean;
var
  node: IXmlNode;
  pl: TPrayList;
  pr: TPrayRec;
begin
  Result := True;
  try
    node := CompDic[Id];
    pl := TPrayList.Create;
    if not node.HasAttribute('mode') then node.SetAttribute('mode', '1');
    _Merge_Instr(node, _notParam, pl, True);
    if pl.Count > 0 then
      begin
        pr := pl[0];
        pr.AdjustFromNode(node, [], '');
        Self.FromRec(pr);
      end;
    pl.Free;
    PrayDic.Add(Id, Self);
  except
    Result := False;
  end;
end;

procedure TPray.AdjustFromNode(const Node: IXmlNode; const Book: string = '');
var
  n, d: string;
begin
  Text := NodeText(Node);
  Title := GetAttr(Node, 'title');
  case Mode of
    0: begin
      Id := FullId(Book, Node.GetAttribute('id'));
      Alias := GetAttr(Node, 'alias');
      StrDivide(GetAttr(Node, 'voice'), n, d, '.');
      if n <> '' then Voice := n.ToInteger else Voice := 0;
      if d <> '' then SubVoice := d.ToInteger else SubVoice := 0;
      PrayType := GetNumAttr(Node, 'type');
      Author := GetNumAttr(Node, 'author');
      if Author = 0 then Author := -GetNumAttr(Node, 'book');
      GloryId := GetAttr(Node, 'glory');
    end;
    1: begin
      Id := Node.GetAttribute('id');
      Alias := Book;
    end;
  end;
end;

function TPray.Load(Id: string): Boolean;
//    
var
  i: Integer;
  book: string;
begin
  StrDivide(Id, book, Id, '.');
  XmlCode.LoadFromFile(GetBookPath(book));
  Result := False;
  with XmlCode.DocumentElement.ChildNodes do
    for i := 0 to Count - 1 do
      if (Nodes[i].NodeName = _xtg[0]) and (GetAttr(Nodes[i], 'id') = Id) then
        begin
          AdjustFromNode(Nodes[i]);
          PrayDic.AddOrSetValue(FullId(book, Id), Self);
          Result := True;
          Break;
        end;
end;

function FindOrLoadPray(Id: string): TPray;
begin
  if PrayDic.TryGetValue(Id, Result) then Exit;
  Result := TPray.Create;
  if not Result.Load(Id) and not Result.TranslateComp(Id) then
    begin
      Result.Free;
      {$IFDEF DEBUG}
      OutMessage(' : '#13#10 + Id);
      {$ENDIF}
    end;
end;

procedure TPrayRec.AdjustFromNode(const Node: IXmlNode; const Params: TStringDynArray; const Book: string = '');
var
  s, n, d: string;
begin
  s := 'id';
  if Id.IsEmpty and Node.HasAttribute(s) then Id := FullId(Book, Node.GetAttribute(s));
  s := 'alias';
  if Node.HasAttribute(s) then Alias := GetAttr(Node, s, Params);
  s := 'title';
  if Node.HasAttribute(s) then Title := GetAttr(Node, s, Params);
  s := 'author';
  n := 'book';
  if Node.HasAttribute(s) then Author := GetNumAttr(Node, s, Params)
  else if Node.HasAttribute(n) then Author := -GetNumAttr(Node, n, Params);
  s := 'voice';
  if Node.HasAttribute(s) then
    begin
      StrDivide(GetAttr(Node, s, Params), n, d, '.');
      Voice := n.ToInteger;
      if d <> '' then SubVoice := d.ToInteger else SubVoice := 0;
    end;
  s := 'type';
  if Node.HasAttribute(s) then PrayType := GetNumAttr(Node, s, Params);
  s := 'role';
  if Node.HasAttribute(s) then SetRoles(GetAttr(Node, s, Params));
  s := 'iter';
  if Node.HasAttribute(s) then Iter := GetNumAttr(Node, s, Params);
  s := 'glory';
  if Node.HasAttribute(s) then GloryId := GetAttr(Node, s, Params);
  if Iter = 0 then Iter := 1;
  s := 'style';
  if Node.HasAttribute(s) then Style := GetNumAttr(Node, s, Params)
  else
    if RolesCount > 0 then
      case Roles[0].Role of
        2: Style := 1;
        3, 5: Style := 2;
        else Style := 0;
      end;
end;

{ TService }

function TService.MiOId: string;
begin
  if FMiOId <> '' then Result := FMiOId
  else
    try
      Result := 'mi_o.' + GloryDic[GloryId].MiOId;
    except
      Result := '';
    end;
end;


function LoadPlaceholders(S: string): TPrayList;
//     
//     
var
  serv, i: Integer;
  p: TPrayRec;
begin
  serv := ToInt(S.Chars[1]);
  S := S.Substring(S.IndexOf('.') + 1);
  S := ServMap[serv].MiOId + '.' + S;
  Result := LoadFromExternService(S, ServMap[serv].Rank);
  for i := 0 to Result.Count - 1 do
    begin
      p := Result[i];
      SetGlory(p, ServMap[serv].GloryId);
      Result[i] := p;
    end;
end;

procedure _Copy_Instr(const Node: IXmlNode; Params: TStringDynArray);
//   copy
var
  count, start, filter, ptype, c, i: Integer;
  spb, dpb: TPrayList;
  s, s2, s3, glory, holi, subscr: string;
  p: TPrayRec;
  b: Boolean;

  procedure _writeAll(const pl: TPrayList);
  var
    i: Integer;
  begin
    for i := start - 1 to spb.Count - 1 do
      begin
        if filter > 0 then
          if spb[i].PrayType <> filter then Continue;
        if filter < 0 then
          if spb[i].PrayType = -filter then Continue;
        p := spb[i];
        if ptype > 0 then p.PrayType := ptype;
        if glory <> '' then p.GloryId := glory;
        if holi <> '' then p.Holi := holi.ToInteger;
        pl.Add(p);
      end;
  end;

  procedure _writeCount(join: Boolean);
  var
    rc, i, j, k: Integer;
    temp: TPrayList;
  begin
    temp := TPrayList.Create;
    _writeAll(temp);
    rc := temp.Count;
    if count <= 0 then count := rc;
    if join and (rc > count) then
      begin
        i := 1;
        while i <= rc do
          begin
            j := rc div count;
            if i <= rc mod count then Inc(j);
            p := temp[i - 1];
            for k := 0 to j - 2 do
              p.Text := p.Text + #13#10 + temp[i + k].Text;
            dpb.Add(p);
            Inc(i, j);
          end;
      end
    else
      begin
        if rc > count then rc := count;
        for i := 1 to rc do
          begin
            j := count div rc;
            if i <= count mod rc then Inc(j);
            for k := 1 to j do
              begin
                p := temp[i - 1];
                dpb.Add(p);
              end;
          end;
      end;
    temp.Free;
  end;

  function _getsub(i: Integer): string;
  var
    g: TGlory;
    n: Integer;
  begin
    if glory <> '' then
      g := GloryDic[glory]
    else
      if spb.Count > 0 then g := GloryDic[spb[0].GloryId]
      else Exit('');
    if g.Grade = 37 then
      case i of
        1: Exit(ApplyLexic('[L:L126:3]', 1));
        2: Exit(ApplyLexic('[L:L21 L11 N29 L30]', 3));
      end;
    if g.Grade = 38 then
      case i of
        1: Exit(ApplyLexic('[L:L128:3]', 1));
        2: Exit(ApplyLexic('[L:L26 G38]', 3));
      end;
    n := g.Stared;
    if (n > 3) or (n = 0) then n := 3;
    Result := '[L:G' + g.Grade.ToString + ':' + n.ToString + ']';
    if n = 1 then
      Result := Result + ' [L:N' + g.FSancts[0].Name.ToString + ']';
    case i of
      1: Exit(ApplyLexic(Result, 2));
      2: Exit(ApplyLexic(Result, 3));
    end;
  end;

  procedure _repl(i: Integer);
  var
    ir: Boolean;
    j: Integer;
    p: TPrayRec;
  begin
    ir := False;
    p := dpb[i];
    for j := 1 to High(ReplList) do
      if (ReplList[j].Id = p.Id)
      and ((ReplList[j].Nomin = 0) or (not p.GloryId.IsEmpty and (ReplList[j].Nomin = GloryDic[p.GloryId].Sancts[1].Nomin))) then
        begin
          ir := True;
          Break;
        end;
    if not ir then Exit;
    with PrayDic[ReplList[j].Val] do
      begin
        p.Id := Id;
        p.Text := Text;
        p.Title := Title;
        p.Author := Author;
        p.Voice := Voice;
        p.SubVoice := SubVoice;
      end;
    dpb.Delete(i);
    dpb.Insert(i, p);
  end;

begin
  start := GetNumAttr(Node, 'start', Params);
  filter := GetNumAttr(Node, 'filter', Params);
  ptype := GetNumAttr(Node, 'type', Params);
  glory := GetAttr(Node, 'glory', Params);
  StrDivide(glory, glory, holi, '.');
  subscr := GetAttr(Node, 'subscr', Params);
  count := GetNumAttr(Node, 'count', Params);
  if start < 1 then start := 1;
  try
    s := GetAttr(Node, 'dest', Params);
    dpb := GetServElm(s, s.Chars[0] <> '^');
    if GetNumAttr(Node, 'del') > 0 then dpb.Clear;
  except
    {$IFDEF DEBUG}
    OutMessage(' : '#13#10 + GetAttr(Node, 'dest'));
    {$ENDIF}
    raise;
  end;
  try
    c := dpb.Count;
    s := GetAttr(Node, 'src', Params);
    case s.Chars[0] of
      '^': begin
        spb := GetServElm(s);
        b := not Assigned(spb) or (spb.Count = 0);
        if b then
          begin
            try
              spb := GetServElm(GetAttr(Node, 'alt', Params));
            except
            end;
            b := ((GetNumAttr(Node, 'mio') + count > 0)) and (not Assigned(spb) or (spb.Count = 0));
            if b then spb := LoadPlaceholders(s);
          end;
        Vars.AddOrSetValue('System.LoadedFromMio', Ord(b).ToString);
      end;
      '!': spb := LoadFromSinglePray(s.Substring(1));
      '@': spb := LoadFromExternService(s.Substring(1), GetNumAttr(Node, 'rank', Params));
      else spb := LoadFromSinglePray(s);
    end;
    while DispatchSubtag('*', subscr, s, s2, s3) do
      subscr := s + _getsub(s2.ToInteger) + s3;
    if count > 0 then _writeCount(GetNumAttr(Node, 'join') > 0) else _writeAll(dpb);
    if (dpb.Count > 0) and not dpb.Last.Id.IsEmpty and (GetNumAttr(Node, 'repl', Params) > 0) then
      for i := dpb.Count - 2 downto 0 do
        if dpb.Last.Id = dpb[i].Id then
          begin
            _repl(i);
            dpb.Remove(dpb.Last);
            Break;
          end;
    if (subscr <> '') and (dpb.Count > c) then
      begin
        p := dpb[c];
        p.Subscr := subscr;
        dpb[c] := p;
      end;
  except
    {$IFDEF DEBUG}
    OutMessage(' : '#13#10'src="' + GetAttr(Node, 'src') + ' (' + s + ')"'#32'dest="' + GetAttr(Node, 'dest') + '"');
    {$ENDIF}
    raise;
  end;
  {$IFDEF DEBUG}
  if b then s := '~> ' else s := '-> ';
  DebugWrite(s + dpb.Count.ToString, True);
  {$ENDIF}
end;

procedure _Del_Instr(const Node: IXmlNode; var Params: TStringDynArray);
//   del
var
  start, ptype, count: Integer;
  spb: TPrayList;
  s: string;
  i: Integer;
begin
  s := GetAttr(Node, 'var', Params);
  if not s.IsEmpty then
    begin
      if s.Chars[0] <> '_' then Vars.Remove(s)
      else
        try
          i := s.Substring(1).ToInteger;
          Params[i] := '';
        except
          SetLength(Params, i + 1);
        end;
      Exit;
    end;
  start := GetNumAttr(Node, 'start', Params);
  count := GetNumAttr(Node, 'count', Params);
  ptype := GetNumAttr(Node, 'type', Params);
  if start < 1 then start := 1;
  try
    s := GetAttr(Node, 'dest', Params);
    if s.Chars[0] <> '^' then s := '^0.' + s;
    spb := GetServElm(s);
    if count = 0 then count := spb.Count;
    for i := start + count - 2 downto start - 1 do
      if ((ptype > 0) and (spb[i].PrayType = ptype)) or (ptype = 0) then
        spb.Delete(i);
  except
    {$IFDEF DEBUG}
    OutMessage('  : '#13#10'del ' + s);
    {$ENDIF}
    raise;
  end;
end;

function TPrayRec.RolesCount: Integer;
begin
  Result := 0;
  for var i in Roles do Inc(Result, i.Count);
end;

function TPrayRec.RolesStr: string;

  function _addOrd(const s: string; i: Integer): string;
  begin
    if Roles[i].Subr = 0 then Result := s
    else Result := s + #32 + CSNum(Roles[i].Subr);
  end;

  function _addCount(const s: string; i: Integer): string;
  begin
    if Roles[i].Subr = 0 then Result := s
    else Result := s + #32 + CSNum(Roles[i].Subr);
  end;

begin
  try
    Result := _addOrd(RolesList[Roles[0].Role].Name[LexCount(Roles[0].Count)], 0);
    for var i := 1 to High(Roles) do
      begin
        if i = 1 then Result := Result + '  '
        else if i = RolesCount - 1 then Result := Result + ' 3 '
        else Result := Result + ', ';
        Result := _addOrd(Result + ApplyTemplate(RolesList[Roles[i].Role], LexCount(Roles[i].Count), 5), i);
      end;
  except
    {$IFDEF DEBUG}
    OutMessage(' : ' + Self.Id);
    {$ENDIF}
    Result := '';
  end;
end;

function UnpackRoles(S: string): TRolesList;
var
  r: StringArray;
begin
  DispatchStrArray(s, r);
  SetLength(Result, Length(r));
  for var i := 0 to Length(r) - 1 do
    begin
      StrDivide(r[i], r[i], s, ':');
      if s.IsEmpty then Result[i].Count := 1 else Result[i].Count := s.ToInteger;
      StrDivide(r[i], r[i], s, '.');
      if s.IsEmpty then Result[i].Subr := 0 else Result[i].Subr := s.ToInteger;
      Result[i].Role := r[i].ToInteger;
    end;
end;

procedure TPrayRec.SetRoles(S: string);
begin
  Roles := UnpackRoles(S);
end;

initialization
  PrayDic := TPrayDic.Create;
  ActDic := TPrayDic.Create;
  ImgDic := TPrayDic.Create;
  PrayList := TList<TPrayRec>.Create;
finalization
  PrayDic.Free;
  ActDic.Free;
  ImgDic.Free;
  PrayList.Free;
end.
