unit EccProcs;

interface

uses
  Types, Classes, EccMemos, EccPrays, EccDefs;

function GetBookPath(const S: string): string;
function StrElm(S: string; B: Byte; const Divider: string = #32): string; inline;
function StrElmCount(S: string; const Divider: string = #32): Integer; inline;
function SubStrCount(const Ss, S: string): Integer; inline;
function StrDivide(S: string; var S1, S2: string; const Ss: string; R: Boolean = False): Boolean;
procedure StrNum2Div(const S: string; var S1: Integer; var S2: string);
function PosExR(const SubStr, S: string; Offset: Cardinal = 0): Integer;
function UnScob(const S: string): string; inline;
function IntVar(const Name: string): Integer; inline;
function BoolVar(const Name: string): Boolean; inline;
function ToInt(const S: string): Integer; inline;
function LastNum(const S: string): Integer; inline;
function CharInString(Ch: Char; const S: string): Boolean; inline;
//      
procedure AddStringList(const SL: TStrings; S: string); inline;
function FullId(const Book, Id: string): string;
function DispatchSubtag(const St, S: string; var S1, S2, S3: string): Boolean;
function DispatchBraces(const S: string; var S1, S2, S3: string): Boolean;
function ApplyFlags(const S: string): string;
function ApplyPV(const S: string; Params: TStringDynArray): string;
function ApplyPrayAll(var P: TPrayRec; Params: TStringDynArray): Boolean;
function ApplyPlaceholders(const P: TPrayRec): string;
function ApplyVars(const S: string): string;
function ApplyLexic(const S: string; P: Integer): string;
function ApplyLists(const S: string): string;
function ApplyPrayGramm(const S: string; Rs, Iter, Sts: Integer): string;
function ApplyObj(const S, Obj: string): string;
procedure InsertInlineAct(var S: string; const Id, Obj: string; I: Integer; M: Integer);
function ValueInStrSet(const V, S: string; const Pre: string = #32; const Post: string = ','): Boolean; inline;
procedure DispatchStrArray(S: string; var A: StringArray; Dv: Char = ',');
function CutLexic(const m: string; v: Char): string;
{$IFDEF DEBUG}
procedure DebugWrite(const s: string; Tab: Boolean = False);
{$ENDIF}

implementation

uses
  SysUtils, StrUtils, UITypes,
  EccVars, EccHtml, EccScripts, EccCSlav, EccLists;

function GetBookPath(const S: string): string;
begin
  Result := GetCurrentDir + '\lib\books\' + S + '.xeds';
end;

function StrElm(S: string; B: Byte; const Divider: string = #32): string; inline;
var
  i: Integer;
begin
  Result := '';
  s := s + Divider;
  try
    for i := 1 to b do
      begin
        TrimLeft(s);
        if s = '' then Exit;
        Result := Copy(s, 1, Pos(Divider, s) - 1);
        System.Delete(s, 1, Length(Result) + Length(Divider));
      end;
  except
    Result := '';
  end;
end;

function StrElmCount(S: string; const Divider: string = #32): Integer; inline;
begin
  Result := 0;
  if S = '' then Exit;
  S := S + Divider;
  while S <> '' do
    begin
      S := S.TrimLeft;
      if S = '' then Exit;
      S := S.Remove(0, S.IndexOf(Divider) + Divider.Length + 1);
      Inc(Result);
    end;
end;

function SubStrCount(const Ss, S: string): Integer; inline;
var
  i: Integer;
begin {opt}
  Result := 0;
  for i := 0 to S.Length - 1 do
    if S.Substring(i, Ss.Length) = Ss then Inc(Result);
end;

function StrDivide(S: string; var S1, S2: string; const Ss: string; R: Boolean = False): Boolean;
var
  i: Integer;
begin {opt}
  if not R then i := S.IndexOf(Ss) else i := S.LastIndexOf(Ss);
  Result := (i >= 0);
  if not Result then i := S.Length;
  S1 := S.Substring(0, i);
  S2 := S.Substring(i + Ss.Length, MaxLongInt);
end;

procedure StrNum2Div(const S: string; var S1: Integer; var S2: string);
begin {opt}
  S1 := S.Substring(1, S.IndexOf('.') - 1).ToInteger;
  S2 := S.Substring(S.IndexOf('.') + 1);
end;

function PosExR(const SubStr, S: string; Offset: Cardinal = 0): Integer;
var
  i, x: Integer;
  lenSubStr: Integer;
begin
  if Offset = 0 then i := S.Length else i := Offset;
  LenSubStr := SubStr.Length;
  while i >= LenSubStr do
  begin
    if S.Chars[i - 1] = SubStr.Chars[lenSubStr - 1] then
    begin
      x := 1;
      while (x < lenSubStr) and (S.Chars[i - x - 1] = SubStr.Chars[lenSubStr - x - 1]) do
        Inc(x);
      if (x = lenSubStr) then
      begin
        Result := (i - lenSubStr) + 1;
        Exit;
      end;
    end;
    Dec(i);
  end;
  Result := 0;
end;

function UnScob(const S: string): string; inline;
begin
  Result := S.Substring(3, S.Length - 4);
end;

function IntVar(const Name: string): Integer;
begin
  Result := Vars[Name].ToInteger;
end;

function BoolVar(const Name: string): Boolean;
begin
  Result := IntVar(Name) = 1;
end;

function ToInt(const S: string): Integer;
begin
  if not TryStrToInt(S, Result) then Result := 0;
end;

function LastNum(const S: string): Integer;
begin
  Result := ToInt(S.Chars[S.Length - 1]);
end;

function CharInString(Ch: Char; const S: string): Boolean;
begin
  Result := S.IndexOf(Ch) > -1;
end;


function FullId(const Book, Id: string): string;
begin
  if (Id.IndexOf('.') = -1) and (IndexStr(Book, [_LoadStart[0], _LoadStart[1], _LoadStart[2]] ) = -1) and (Book <> '') then
    Result := Book +'.' + Id
  else
    Result := Id;
end;

procedure AddStringList(const SL: TStrings; S: string);
var
  s1: string;
  b: Boolean;
begin
  repeat
    b := StrDivide(S, s1, S, #10);
    SL.Add(s1);
  until not b;
end;

function ApplyParams(S: string; Params: TStringDynArray): string;
var
  i, j: Integer;
begin
  i := S.IndexOf('[_');
  while i >= 0 do
    begin
      Result := S.Substring(i + 2);
      j := Result.Substring(0, Result.IndexOf(']')).ToInteger;
      try
        Result := Params[j - 1];
      except
        Result := '';
      end;
      S := S.Replace('[_' + j.ToString + ']', Result, [rfReplaceAll]);
      i := S.IndexOf('[_');
    end;
  Result := S;
end;

function DispatchSubtag(const St, S: string; var S1, S2, S3: string): Boolean;
var
  i, c: Integer;
begin
  Result := StrDivide(S, S1, S2, '[' + St + ':');
  if not Result then Exit;
  c := 0;
  for i := 0 to S2.Length - 1 do
    case S2.Chars[i] of
      ']': begin
        if c = 0 then Break;
        Dec(c);
      end;
      '[': Inc(c);
    end;
  S3 := S2.Substring(i + 1);
  S2 := S2.Substring(0, i);
end;

function DispatchBraces(const S: string; var S1, S2, S3: string): Boolean;
var
  i, c: Integer;
begin
  Result := StrDivide(S, S1, S2, '(');
  if not Result then Exit;
  c := 0;
  for i := 0 to S2.Length - 1 do
    case S2.Chars[i] of
      ')': begin
        if c = 0 then Break;
        Dec(c);
      end;
      '(': Inc(c);
    end;
  S3 := S2.Substring(i + 1);
  S2 := S2.Substring(0, i);
end;

function ApplyLexic(const S: string; P: Integer): string;
var
  s1, s2, s3, s4: string;
  i, c, pp: Integer;
  l: LexicArray;
begin
  Result := ApplyVars(S);
  if P = 0 then Exit;
  while DispatchSubtag('L', Result, s1, s3, Result) do
    begin
      s4 := '';
      if s3 <> '' then
        begin
          s3 := s3.Trim + #32;
          repeat
            s2 := s3.Substring(0, Pos(#32, s3) - 1);
            s3 := s3.Substring(Pos(#32, s3));
            c := 1;
            pp := P;
            if Pos(':', s2) <> 0 then
              begin
                c := LastNum(s2);
                if (c = 2) and not _s_ForceDouble then c := 3;
                if c > 3 then {  }
                  begin
                    if (P = 1) or (P = 4) then
                      begin
                        if c = 4 then c := 1;
                        pp := 2;
                      end;
                     if c > 3 then c := 3;
                  end;
                s2 := s2.Substring(0, s2.Length - 2);
              end;
            i := s2.Substring(1).ToInteger;
            l := GetLexicArray(s2.Chars[0]);
            s4 := s4 + ' ' + ApplyTemplate(l[i].Name[c], l[i].Template[c], pp);
          until s3 = '';
        end;
      Result := s1 + s4.Trim + Result;
    end;
end;

function ApplyVars(const S: string): string;
var
  s1, s2, s3: string;
begin
  Result := S;
  while DispatchSubtag('V', Result, s1, s2, Result) do
    begin
      s2 := ApplyVars(s2);
      if CharInSet(s2.Chars[0], ['A'..'Z']) then
        try
          if s2[Length(s2) - 1] = ':' then
            s3 := ApplyLexic(Vars.Items[Copy(s2, 1, Length(s2) - 2)], LastNum(s2))
          else s3 := Vars.Items[s2];
        except
          {$IFDEF DEBUG}
{          OutMessage(' : ' + s2 + #13#10' :'#13#10 + S);}
          {$ENDIF}
          s3 := '';
        end;
      Result := s1 + s3 + Result;
    end;
end;

function CutDispatchIndex(var S: string): Integer;
var
  l: string;
begin
  StrDivide(S, S, l, '[');
  if not l.IsEmpty then l := l.Substring(0, l.Length - 1);
  if l.Length > 0 then
    begin
      Result := -('SMWL'.IndexOf(l.Chars[0]) + 1);
      if Result = 0 then Result := l.ToInteger;
    end
  else Result := 0;
end;

procedure InitInterval(var m: Integer; C: Integer; out st, en: Integer);
begin
  st := 0;
  en := c;
  case m of
    0: ;
    -1: st := 1;
    -2: begin st := 1; Dec(en); end;
    -3: Dec(en);
    -4: st := en;
    else begin
      if m > c + 1 then m := c + 1;
      st := m - 1; en := m - 1;
    end;
  end;
end;

function ApplyLists(const S: string): string;
var
  s1, s2, s3, s4, dv: string;
  sl: TStringList;
  dic: TPersonDic;
  i, p, m, st, en, n, e: Integer;

procedure _dispListParams(var s: string; out p: Integer; out dv: string);
var
  l: string;
begin
  StrDivide(s, s, l, ':');
  if l.IsEmpty then p := 0
  else
    begin
      p := ToInt(l.Chars[0]);
      if p < 0 then p := 0 else l := l.Substring(1, MaxLongInt);
    end;
  StrDivide(l, l, Dv, '[');
  if dv <> '' then
    dv := dv.Substring(0, dv.Length - 1).Replace('*', #13#10)
  else
    dv := ', ';
end;

begin
  Result := S;
  sl := TStringList.Create;
  while DispatchSubtag('S', Result, s1, s2, Result) do
    begin
      s3 := '';
      s2 := ApplyVars(s2);
      _dispListParams(s2, p, dv);
      if CharInSet(s2.Chars[0], ['A'..'Z']) then
        try
          s4 := s2.Substring(s2.LastIndexOf('.') + 1, MaxLongInt);
          case IndexStr(s4, ['Count', 'LexCount', 'GendCount']) of
            0: begin
              sl.Text := Vars[s2.Substring(0, s2.Length - s4.Length - 1)];
              s3 := sl.Count.ToString;
            end;
            1: begin
              sl.Text := Vars[s2.Substring(0, s2.Length - s4.Length - 1)];
              s3 := LexCount(sl.Count).ToString;
            end;
            2: begin
              i := 0; e := 0;
              for st := 0 to sl.Count - 1 do
                if NamesList[CutLexic(sl[st], 'N').ToInteger].Gen = 2 then Inc(i) else Inc(e);
              s3 := (LexCount(sl.Count) + Ord((i = 0) and (e > 0)) * 3).ToString;
            end;
            else begin
              m := CutDispatchIndex(s2);
              sl.Text := Vars[s2];
              InitInterval(m, sl.Count - 1, st, en);
              for i := st to en do
                s3 := s3 + ApplyLexic(sl[i], p) + dv;
              SetLength(s3, s3.Length - dv.Length);
            end;
          end;
        except
          {$IFDEF DEBUG}
          OutMessage('  : ' + s2 + #13#10' :'#13#10 + S);
          {$ENDIF}
        end;
      Result := s1 + s3 + Result;
    end;
  sl.Free;
  while DispatchSubtag('X', Result, s1, s2, Result) do
    begin
      s2 := ApplyVars(s2);
      if CharInSet(s2.Chars[0], ['A'..'Z']) then
        try
          _dispListParams(s2, p, dv);
          s4 := s2.Substring(s2.LastIndexOf('.') + 1, MaxLongInt);
          e := IndexStr(s4, ['Titul', 'Call', 'Full', 'Count']);
          s2 := s2.Substring(0, s2.Length - s4.Length - 1);
          m := CutDispatchIndex(s2);
          s2 := Trim(Vars[s2]);
          s3 := '';
          i := SubStrCount(',', s2);
          if e = 3 then
            if Trim(s2).IsEmpty then s3 := '0'
            else s3 := (i + 1).ToString
          else if s2 <> '' then
            begin
              InitInterval(m, i, st, en);
              for i := 0 to en do
                begin
                  StrDivide(s2, s4, s2, ',');
                  s2 := s2.Trim;
                  if i < st then Continue;
                  if s4.Chars[0] in ['0'..'9', '-'] then
                    begin
                      n := s4.ToInteger;
                      if n < 0 then dic := PersonsDic else dic := SanctsDic;
                      n := Abs(n);
                      case e of
                        0: s4 := dic[n].Title;
                        1: s4 := dic[n].Call;
                        2: s4 := dic[n].Full;
                      end;
                      s4 := ApplyLexic(s4, p);
                    end
                  else
                    begin
                      case e of
                        0: s4 := GloryDic[s4].Title;
                        1: s4 := GloryDic[s4].Call;
                        2: s4 := GloryDic[s4].Full;
                      end;
                      s4 := ApplyLexic(s4, p);
                    end;
                  s3 := s3 + s4 + dv;
                end;
              SetLength(s3, Length(s3) - Length(dv));
            end;
        except
          {$IFDEF DEBUG}
          OutMessage('  : ' + s2 + #13#10' :'#13#10 + S);
          {$ENDIF}
        end;
      Result := s1 + s3 + Result;
    end;
end;

function ApplyPrayAll(var P: TPrayRec; Params: TStringDynArray): Boolean;
var
  n: Integer;
begin
  try
    n := GloryDic[P.GloryId].Stared;
  except
    n := 0;
  end;
  P.Text := ApplyPrayGramm(ApplyPV(ApplyFlags(ApplyPlaceholders(P)), Params), P.RolesCount, P.Iter, n);
  P.Mode := 0;
  Result := P.Text <> '';
end;

function ApplyPlaceholders(const P: TPrayRec): string;
var
  i, j, d, st, en: Integer;
  s1, s2, s3: string;
  g: TGlory;
begin
  Result := P.Text;
  while DispatchSubtag('P', Result, s1, s2, Result) do
    begin
      g := GloryDic[P.GloryId];
      StrDivide(s2, s2, s3, ':');
      d := CutDispatchIndex(s2);
      case IndexStr(s2, ['NameList', 'Call', 'Holiday', 'Nominator', 'Gend']) of
        0: begin
          s2 := '';
          i := s3.ToInteger;
          InitInterval(d, g.Stared - 1, st, en);
          for j := st to en do
            s2 := s2 + ApplyTemplate(NamesList[g.FSancts[j].Name], 1, i) + ', ';
          s2 := s2.Substring(0, s2.Length - 2);
        end;
        1: s2 := ApplyLexic(g.Call, s3.ToInteger);
        2: s2 := ApplyTemplate(HolidaysList[P.Holi], 1, s3.ToInteger);
        3: begin
          if s3.IndexOf(':') >= 0 then
            begin
              j := ToInt(s3.Chars[0]);
              s3 := s3.Chars[2];
            end
          else j := LexCount(g.Stared);
          s2 := ApplyTemplate(GradesList[g.Grade], j, s3.ToInteger);
        end;
        4: s2 := GradesList[g.Grade].Gen.ToString;
        {$IFDEF DEBUG}
        else OutMessage('  :'#13#10 + s2);
        {$ENDIF}
      end;
      Result := s1 + s2 + Result;
    end;
end;

function ApplyFlags(const S: string): string;
var
  s1, s2, f: string;
  _junc: TStringDynArray;
begin
  Result := S;
  while DispatchSubtag('F', Result, s1, s2, Result) do
    begin
      s2 := ApplyPV(s2, _junc);
      StrDivide(s2, f, s2, #32);
      if f = '0' then Result := s1 + Result else Result := s1 + s2 + Result;
    end;
end;

function ApplyPV(const S: string; Params: TStringDynArray): string;
var
  s1, s2, e: string;
begin
  Result := S;
  while DispatchBraces(Result, s1, e, s2) do
    Result := s1 + CalcExpression(e, Params) + s2;
  Result := ApplyLists(ApplyVars(ApplyParams(Result, Params)));
  while DispatchSubtag('N', Result, s1, s2, Result) do
    Result := s1 + CSNum(s2.ToInteger) + Result;
  while DispatchSubtag('C', Result, s1, s2, Result) do
    Result := s1 + FirstCap(s2) + Result;
end;

function ApplyPrayGramm(const S: string; Rs, Iter, Sts: Integer): string;
var
  s1, s2, s3, s4: string;

  function _getsub(const s: string; n: Integer): string;
  var
    a: StringArray;
  begin
    Result := s;
    try
      DispatchStrArray(s, a, '/');
      if Length(a) in [3, 6] then
        begin
          if not _s_ForceDouble and (n in [2, 5]) then Inc(n);
          if n = 0 then n := 3;
        end
      else if n = 0 then n := 1;
      Result := a[n - 1];
    finally
      SetLength(a, 0);
    end;
  end;

  function _prayParam(s: string): Integer;
  var
    i: Integer;
    s1: string;
  begin
    try
      Exit(s.ToInteger);
    except
      DispatchSubtag('I', s, s1, s, s1);
        case IndexStr(s, ['Role', 'Iter', 'LexCount']) of
          0: i := Rs;
          1: i := Iter;
          2: i := Sts;
          else
            begin
              {$IFDEF DEBUG}
              OutMessage('  :'#13#10 + s);
              {$ENDIF}
              Exit(1);
            end;
        end;
      Result := LexCount(i);
    end;
  end;

begin
  Result := S;
  while DispatchSubtag('G', Result, s1, s2, s3) do
    begin
      StrDivide(s2, s4, s2, #32);
      Result := s1 + _getsub(s2, _prayParam(ApplyVars(s4))) + s3;
    end;
end;

function ApplyObj(const S, Obj: string): string;
var
  s1, s2, s3: string;
begin
  DispatchSubtag('*', S, s1, s2, s3);
  if not s2.IsEmpty then s2 := ApplyLexic(Obj, s2.ToInteger);
  Result := s1 + s2 + s3;
end;

procedure InsertInlineAct(var S: string; const Id, Obj: string; I: Integer; M: Integer);

  function _s(const Id, Obj: string; I: Integer): string;
  begin
    Result := Id;
    if not Obj.IsEmpty then Result := Result + '[' + Obj.Substring(3, Obj.Length - 3);
    if I > 1 then Result := Result + ':' + I.ToString;
  end;

  function _l(const Id, Obj: string; I: Integer): string;
  begin
    Result := '[A:' + _s(Id, Obj, I) + ']';
  end;

  function _istag(const S: string; I: Integer): Boolean;
  var
    j, c: Integer;
  begin
    Result := False;
    if S.Chars[I] <> ']' then Exit;
    c := 0;
    for j := I - 1 downto 0 do
      case S.Chars[j] of
        '[': begin
          if c = 0 then Break;
          Dec(c);
        end;
        ']': Inc(c);
      end;
    Exit(S.IndexOf('[A:', j) = j);
  end;

begin
  case M of
    -1: S := _l(Id, Obj, I);
    1: begin
      if _istag(S, S.Length - 2) then
        S := S.Insert(S.Length - 2, ',' + _s(Id, Obj, I))
      else
        S := S.Insert(S.Length - 1, _l(Id, Obj, I));
    end;
    0: begin
      if _istag(S, S.Length - 1) then
        S := S.Insert(S.Length - 1, ',' + _s(Id, Obj, I))
      else
        S := S + _l(Id, Obj, I);
    end;
  end;
end;

function ValueInStrSet(const V, S: string; const Pre: string = #32; const Post: string = ','): Boolean;
begin
  Result := ((Pre + S + Post).IndexOf(Pre + V + Post) >= 0);
end;

procedure DispatchStrArray(S: string; var A: StringArray; Dv: Char = ',');
begin
  SetLength(A, 0);
  S := Trim(S);
  if S.IsEmpty then Exit;
  SetLength(A, S.CountChar(Dv) + 1);
  for var i := 0 to Length(A) - 1 do
    begin
      StrDivide(S, A[i], S, Dv);
      S := Trim(S);
    end;
end;

function CutLexic(const m: string; v: Char): string;
begin
  Result := m.Substring(m.IndexOf(v));
  Result := Result.Substring(1, Result.IndexOfAny([#32, ']']) - 1);
end;

{$IFDEF DEBUG}
var
  _f: Boolean = True;

procedure DebugWrite(const s: string; Tab: Boolean = False);
var
  f: Text;
  i: Integer;
begin
  AssignFile(f, 'debug.txt');
  if _f then
    begin
      Rewrite(f);
      _f := False;
    end
  else
    Append(f);
  if Tab then for i := 1 to _NodeLevel do Write(f, #32#32);
  WriteLn(f, s);
  CloseFile(f);
end;
{$ENDIF}

end.
