unit EccReqst;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Objects;

type
  TReqFrame = class(TFrame)
    Button1: TButton;
    CheckBox1: TCheckBox;
    Label1: TLabel;
    Panel1: TPanel;
    Rectangle1: TRectangle;
    procedure Button1Click(Sender: TObject);
  private
    function ContentHeight: Single;
    procedure Implement;
    procedure SetAndSave(const VarName, Value: string);
  public
    procedure Prepare(const Title: string; Save: Boolean);
    procedure AddText(const Str: string);
    procedure AddRadioGroup(const Str, VarName: string; Def: Integer);
    procedure AddCheckBox(const Str, VarName: string; Def: Boolean);
    procedure ShowModal;
  end;

implementation

uses
  EcclysiarchMain, EccDefs, EccProcs, EccVars;

var
  Waiting: Boolean;

{$R *.fmx}

procedure AddFloatControl(const S, C: TControl);
begin
  C.Position.Y := 1000;
  C.Align := TAlignLayout.Top;
  S.AddObject(C);
end;

function TReqFrame.ContentHeight: Single;
var
  l: TControl;
begin
  Result := 0;
  for l in Panel1.Controls do Result := Result + l.Height;
end;

procedure TReqFrame.AddCheckBox(const Str, VarName: string; Def: Boolean);
var
  l: TCheckBox;
begin
  l := TCheckBox.Create(Panel1);
  l.WordWrap := True;
  l.Text := Str;
  l.StyledSettings := [TStyledSetting.Style];
  l.Font.Size := 17;
  l.Margins.Left := 12;
  l.Height := 24;
  l.FontColor := $FFCFCFCF;
  l.IsChecked := Def;
  l.TagString := VarName;
  AddFloatControl(Panel1, l);
end;

procedure TReqFrame.AddRadioGroup(const Str, VarName: string; Def: Integer);
var
  l: TGroupBox;
  r: TRadioButton;
  A: StringArray;
  i: Integer;
begin
  DispatchStrArray(Str, A, '/');
  l := TGroupBox.Create(Panel1);
  l.Text := #32 + A[0] + #32;
  l.StyledSettings := [TStyledSetting.Style];
  l.Font.Size := 17;
  l.FontColor := $FF66F8E5;
  l.Padding.Left := 20;
  l.Margins.Top := 4;
  l.Padding.Top := 24;
  l.TagString := VarName;
  for i := 1 to High(A) do
    begin
      r := TRadioButton.Create(l);
      r.Font.Size := 17;
      r.Text := A[i];
      r.Height := 24;
      r.IsChecked := (i = Def + 1);
      r.GroupName := VarName;
      r.FontColor := $FFCFCFCF;
      r.StyledSettings := [TStyledSetting.Style];
      AddFloatControl(l, r);
    end;
  AddFloatControl(Panel1, l);
  l.Height := l.ChildrenCount * 24 + 40;
  SetLength(A, 0);
end;

procedure TReqFrame.AddText(const Str: string);
var
  l: TLabel;
begin
  l := TLabel.Create(Panel1);
  l.WordWrap := True;
  l.StyledSettings := [TStyledSetting.Style, TStyledSetting.FontColor];
  l.Font.Size := 17;
  l.Padding.Top := 6;
  l.AutoSize := True;
  l.Text := Str;
  AddFloatControl(Panel1, l);
  l.RecalcSize;
end;

procedure TReqFrame.Button1Click(Sender: TObject);
begin
  Implement;
  Visible := False;
  EccMainForm.TabControl2.ActiveTab := EccMainForm.TabItem2;
  EccMainForm.Panel4.Enabled := True;
  Waiting := False;
end;

procedure TReqFrame.SetAndSave(const VarName, Value: string);
begin
  Vars.AddOrSetValue(VarName, Value);
  if CheckBox1.IsChecked then
    begin
      StoreVar(VarName, Value, Label1.Text);
      RqVars.AddOrSetValue(VarName, Label1.Text);
    end;
end;

procedure TReqFrame.Implement;
var
  l, j: TControl;
begin
  for l in Panel1.Controls do
    if l.ClassType = TGroupBox then
      for j in l.Controls do
        if TRadioButton(j).IsChecked then
          SetAndSave(l.TagString, (j.Index - 1).ToString)
        else
    else if l.ClassType = TCheckBox then
      SetAndSave(l.TagString, Ord(TCheckBox(l).IsChecked).ToString);
end;

procedure TReqFrame.Prepare(const Title: string; Save: Boolean);
begin
  Panel1.DeleteChildren;
  CheckBox1.IsChecked := False;
  CheckBox1.Visible := Save;
  Label1.Text := Title;
  Button1.SetFocus;
end;


procedure TReqFrame.ShowModal;
begin
  try
    EccMainForm.Panel4.Enabled := False;
    try
      EccMainForm.Switch;
    except
    end;
    Waiting := True;
    Visible := True;
    Panel1.Height := ContentHeight + 18;
    Height := Trunc(Panel1.Height + 62 + Label1.Height);
    repeat Application.ProcessMessages until not Waiting;
  except
    Button1Click(nil);
    raise;
  end;
end;

end.
