unit EccScripts;

interface

uses
  XmlIntf, Generics.Collections, Types,
  EccDefs, EccMemos, EccPrays;

type
  TNodeList = TList<IXmlNode>;
  TXmlDic = TDictionary<string, IXmlNode>;

  TMacroParams = packed record
    Cond, Brk, Ret: Boolean;
  end;

procedure ProcessNode(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; var MP: TMacroParams);
function ProcessChild(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList): Boolean;
procedure LoadDics(const Book: string);
procedure _Seq_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; IsRite: Boolean = False);
procedure _Merge_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; Br: Boolean = False);
procedure Request(const Node: IXmlNode);

var
  XmlCode: IXmlDocument;
  CompDic, MacroDic, StructDic, ProcDic, ChainDic, RitesDic,
    AssemDic: TXmlDic;
  RequestVariants: procedure(const Node: IXmlNode; const s: string);

const
  _xtg: array[0..17] of string =
    ('pray', '', 'act', 'macro', 'proc', 'rite',
    'comp', 'assem', 'glory', 'pers', 'sanct', 'date',
    'var', 'list', 'pomest', 'struct', 'chain', 'img');
  _xstg: array[0..30] of string =
    ('if', 'calc', 'sel', 'seq', 'sync', 'merge', 'inter',
    'copy', 'text', '', 'add', 'addall', 'extr', 'act',
    'macro', 'proc', 'rept', 'while', 'else', 'brif', 'ret',
    'rite', 'rqst', 'chain', 'test', '', '',
    'title', 'del', 'join', 'img');

implementation

uses
  StrUtils, SysUtils, Variants, Classes, xmldom, XMLDoc,
  EccProcs, EccXml, EccCSlav, EccLists, EccHtml, EccAdmin,
  EccVars;

procedure _Incl_Instr(const Id: string; const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; const Dic: TXmlDic);
var
  sendparams: TStringDynArray;
  s: string;
  j: Integer;
  proc: Boolean;
begin
  j := 1;
  proc := Node.NodeName = 'proc';
  while Node.HasAttribute('_' + IntToStr(j)) or (j <= 2) do
    begin
      SetLength(sendparams, j);
      sendparams[j - 1] := GetAttr(Node, '_' + IntToStr(j), Params);
      Inc(j);
    end;
  try
    if ProcessChild(Dic.Items[Id], sendparams, List) and proc then
      begin
        s := GetAttr(Node, 'ret', Params);
        if s <> '' then
          begin
            if s[1] = '_' then
              begin
                j := StrToInt(Copy(s, 2, Length(s)));
                if Length(Params) < j then SetLength(Params, j);
                Params[j - 1] := sendparams[0];
              end
            else
              Vars.AddOrSetValue(s, sendparams[0]);
          end;
      end;
  except
    {$IFDEF DEBUG}
    OutMessage('   (  ): '#13#10 + Id);
    {$ENDIF}
  end;
end;

procedure _Sync_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList);
var
  j: Integer;
  p: TPrayRec;
  l: TPrayList;
  _junk: TMacroParams;
begin
  p.Mode := 2;
  p.Items := TList<TList<TPrayRec>>.Create;
  Inc(_NodeLevel);
  for j := 0 to Node.ChildNodes.Count - 1 do
    begin
      l := TList<TPrayRec>.Create;
      p.Items.Add(l);
      ProcessNode(Node.ChildNodes[j], Params, l, _junk);
    end;
  List.Add(p);
  Dec(_NodeLevel);
end;

procedure _Join_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList);
var
  i, j, k: Integer;
  p: TPrayRec;
  l0, l: TPrayList;
begin
  l0 := TPrayList(List.Last.Items.Last);
  j := GetNumAttr(Node, 'count', Params);
  k := GetNumAttr(Node, 'start', Params);
  if (j = 0) or (k = 0) then
    ProcessChild(Node, Params, l0)
  else
    begin
      if k > 0 then k := k - 1 else k := l0.Count + k - j + 1;
      if j < 0 then j := l0.Count - k + j + 1;
      p.Mode := 2;
      p.Items := TList<TList<TPrayRec>>.Create;
      l := TList<TPrayRec>.Create;
      p.Items.Add(l);
      for i := 1 to j do
        begin
          l.Add(l0[k]);
          l0.Delete(k);
        end;
      l := TList<TPrayRec>.Create;
      p.Items.Add(l);
      ProcessChild(Node, Params, l);
      l0.Insert(k, p);
    end;
end;

procedure _Seq_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; IsRite: Boolean = False);
var
  j: Integer;
  p: TPrayRec;
  l: TPrayList;
begin
  if IsRite then j := 3 else j := GetNumAttr(Node, 'lev', Params);
  if j = 0 then ProcessChild(Node, Params, List)
  else
    begin
      p.Mode := 2;
      p.Text := ApplyPV(ApplyFlags(GetAttr(Node, 'title')), Params);
      if IsRite then p.Iter := 1 else P.Iter := GetNumAttr(Node, 'exp');
      //  Iter   
      p.Style := j;
      p.Items := TList<TList<TPrayRec>>.Create;
      l := TList<TPrayRec>.Create;
      p.Items.Add(l);
      ProcessChild(Node, Params, l);
      List.Add(p);
    end;
end;

function GetDivider(const Node: IXmlNode; var Params: TStringDynArray): string;
begin
  if GetNumAttr(Node, 'break', Params) = 1 then Exit(#13#10);
  if GetNumAttr(Node, 'red', Params) = 1 then Exit(#9);
  Result := #32;
end;

function Mutliple(const p: TPrayRec; const dv: string): TPrayRec;
begin
  Result := p;
  Result.Iter := 1;
  for var i := 2 to p.Iter do
    Result.Text := Result.Text + dv + p.Text;
end;

procedure _Merge_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; Br: Boolean = False);
  label _procend_;
  var
    j: Integer;
    s, m, dv: string;
    p: TPrayRec;
begin
  j := List.Count;
  if Br then dv := #13#10 else dv := GetDivider(Node, Params);
  ProcessChild(Node, Params, List);
  if List.Count <= j then Exit;
  p := Mutliple(List[j], dv);
  s := '';
  m := '';
  while p.Mode = 1 do
    begin
      m := m + ', ' + p.Text;
      if s.IsEmpty then
        InsertInlineAct(s, List.Items[j].Id, List.Items[j].Alias, List.Items[j].Iter, -1)
      else
        InsertInlineAct(s, List.Items[j].Id, List.Items[j].Alias, List.Items[j].Iter, 0);
      List.Delete(j);
      if List.Count > j then p := Mutliple(List[j], dv)
      else
        begin
          p.Text := m.Substring(2);
          goto _procend_;
        end;
    end;
  List.Delete(j);
  while List.Count > j do
    begin
      case List.Items[j].Mode of
        0: p.Text := p.Text + dv + Mutliple(List.Items[j], dv).Text;
        1: InsertInlineAct(p.Text, List.Items[j].Id, List.Items[j].Alias, List.Items[j].Iter, Ord((dv = #32) or (p.Iter = 1)));
      end;
      List.Delete(j);
    end;
  if not s.IsEmpty then p.Text := s + #32 + p.Text.Replace(#13#10, #32, [rfReplaceAll]);
  if dv = #32 then p.Text := p.Text.Replace(#13#10, #32, [rfReplaceAll]);
  p.Mode := 0;
  _procend_: p.AdjustFromNode(Node, Params, '');
  List.Add(p);
end;

procedure _Rep_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList);
var
  s: string;
  st, fi, i: Integer;
begin
  s := 'Cycle._' + GetAttr(Node, 'id', Params);
  st := GetNumAttr(Node, 'start', Params);
  fi := GetNumAttr(Node, 'end', Params);
  if fi >= st then
    for i := st to fi do
      begin
        Vars.AddOrSetValue(s, IntToStr(i));
        ProcessChild(Node, Params, List);
      end
  else
    if GetAttr(Node, 'rev') = '1' then
      for i := st downto fi do
        begin
          Vars.AddOrSetValue(s, IntToStr(i));
          ProcessChild(Node, Params, List);
        end;
end;

procedure _Inter_Instr(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList);
var
  i, j, k, c, n, x: Integer;
  mode, dv: string;
  p, w: TPrayRec;
  sl, dl: TStringList;
  _junk: TMacroParams;
  il, join, mirr: Boolean;

  procedure _applyRole(var p: TPrayRec; c: Integer);
  begin
    if not il then Exit;
    if P.RolesCount > 0 then
      if _o_TwoLiks and (p.Roles[0].Role = 2) then
        p.Roles[0].Subr := 1 + c;
  end;

  procedure _applyPray(n, c: Integer);
  var
    p: TPrayRec;
  begin
    if not il then Exit;
    p := List[n];
    _applyRole(p, c);
    List[n] := p;
  end;

  procedure _mirror(var p, w: TPrayRec);
  begin
    w := List[i];
    p.Roles := w.Roles;
    p.Style := w.Style;
    p.Voice := w.Voice;
    p.SubVoice := w.SubVoice;
    p.Subscr := w.Subscr;
    w.Subscr := '';
    List[i] := w;
  end;

begin
  Inc(_NodeLevel);
  c := List.Count + 1;
  ProcessNode(Node.ChildNodes[0], Params, List, _junk);
  p := List[c - 1];
  dv := GetDivider(Node, Params);
  p := Mutliple(p, dv);
  sl := TStringList.Create;
  sl.Text := p.Text;
  n := sl.Count;
  mode := GetAttr(Node, 'mode');
  il := GetNumAttr(Node, 'incr') = 1;
  join := (GetNumAttr(Node, 'join') = 1) or (dv = #9) or (dv = #13#10);
  mirr := GetNumAttr(Node, 'mirr') = 1;
  case IndexStr(mode, ['stih', 'ypo', 'anti', 'circ', 'ins']) of
    0: begin
      for i := 1 to Node.ChildNodes.Count - 1 do
        begin
          k := GetNumAttr(Node.ChildNodes[i], 'iter');
          if k < 1 then k := 1;
          for j := 1 to k do
            begin
              ProcessNode(Node.ChildNodes[i], Params, List, _junk);
              p := List[List.Count - 1];
              p.Iter := 1;
              List[List.Count - 1] := p;
            end;
          for j := c to List.Count - 1 do
            _applyPray(j, (j - c) mod 2);
        end;
      for i := List.Count - 1 downto c do
        begin
          if n = 0 then Break;
          if not join then
            begin
              p.Text := sl[n - 1];
              p.PrayType := List[c - 1].PrayType;
              if mirr then _mirror(p, w);
              List.Insert(i, p);
            end
          else
            begin
              p := List[i];
              p.Text := sl[n - 1] + dv + p.Text;
              _applyRole(p, (i - c) mod 2);
              List[i] := p;
            end;
          Dec(n);
        end;
      dv := GetAttr(Node, 'subscr', Params);
      if dv > '' then
        begin
          p := List[c];
          if p.Subscr <> '' then p.Subscr := dv + #32 + p.Subscr
          else p.Subscr := dv;
          List[c] := p;
        end;
      if n > 0 then
        for i := 0 to n - 1 do
          begin
            p.Text := sl[i];
            p.Roles := List[c - 1].Roles;
            _applyRole(p, i mod 2);
            p.Subscr := '';
            p.PrayType := 0;
            List.Insert(c + i, p);
          end;
    end;
    1: for i := 0 to n - 1 do
      if join then
        begin
          ProcessNode(Node.ChildNodes[1], Params, List, _junk);
          p.Text := sl[i];
          if List[List.Count - 1].Mode = 1 then
            InsertInlineAct(p.Text, List[List.Count - 1].Id, List[List.Count - 1].Alias, List[List.Count - 1].Iter, 1)
          else
            p.Text := p.Text + dv + List[List.Count - 1].Text;
          p.Roles := List[c - 1].Roles;
          _applyRole(p, i mod 2);
          List[List.Count - 1] := p;
        end
      else
        begin
          p.Text := sl[i];
          p.Roles := List[c - 1].Roles;
          _applyRole(p, i mod 2);
          List.Add(p);
          k := Node.ChildNodes.Count;
          for j := 1 to k - 1 do
            ProcessNode(Node.ChildNodes[j], Params, List, _junk);
          _applyPray(List.Count - 1, i mod 2);
        end;
    2: begin
      for i := 0 to n - 1 do
        begin
          p.Text := sl[i];
          p.AdjustFromNode(Node, Params, '');
          _applyRole(p, i mod 2);
          List.Add(p);
          p.AdjustFromNode(Node.ChildNodes[0], Params, '');
          _applyRole(p, i mod 2);
          List.Add(p);
        end;
    end;
    3: begin
      k := Node.ChildNodes.Count - 1;
      var r := List[c - 1].Roles;
      for i := -1 to n - 1 do
        begin
          if i >= 0 then
            begin
              p.Text := sl[i];
              p.Roles := r;
              _applyRole(p, (i + 1) mod 2);
              List.Add(p);
            end;
          ProcessNode(Node.ChildNodes[(i + 1) mod k + 1], Params, List, _junk);
          p.Voice := List[List.Count - 1].Voice;
          List[List.Count - 2] := p;
          _applyPray(List.Count - 1, (i + 1) mod 2);
        end;
    end;
    4: begin
      k := Node.ChildNodes.Count - 1;
      for i := 1 to k do
        ProcessNode(Node.ChildNodes[i], Params, List, _junk);
      k := List.Count - 1;
      for i := c to k do
        begin
          dl := TStringList.Create;
          dl.Text := List[i].Text;
          if n > dl.Count then n := dl.Count;
          for j := 0 to n - 1 do
            sl[j] := sl[j] + dv + dl[j];
          dl.Free;
        end;
      while List.Count > c do List.Delete(List.Count - 1);
      while sl.Count > n do sl.Delete(sl.Count - 1);
      p.Text := sl.Text;
      List.Add(p);
    end;
    {$IFDEF DEBUG}
    else OutMessage('   <inter>: ' + mode);
    {$ENDIF}
  end;
  List.Delete(c - 1);
  sl.Free;
  Vars.AddOrSetValue('System.LastInterRole', List[List.Count - 1].Roles[0].Subr.ToString);
  Dec(_NodeLevel);
end;

procedure ProcessRite(const Id: string; PL: TPrayList);
var
  Params: TStringDynArray;
  a: StringArray;
begin
  Vars.AddOrSetValue('Now.Rite.Id', Id);
  Vars.AddOrSetValue('Now.Rite.Class', GetAttr(RitesDic.Items[Id], 'class', Params));
  Vars.AddOrSetValue('Now.Rite.Cycle', GetAttr(RitesDic.Items[Id], 'cycle', Params));
  DispatchStrArray(GetAttr(RitesDic.Items[Id], 'show', Params), a);
  for var i := 0 to Length(a) - 1 do
    if Vars['Show.' + a[i]] = '0' then Exit;
  SetLength(Params, 0);
  _Seq_Instr(RitesDic.Items[Id], Params, PL, True);
end;

function ErrInfo(const Node: IXmlNode; var Params: TStringDynArray): string;
var
  attr: IXmlNodeList;
  i: Integer;
begin
  Result := '  :'#13#10'  <' + Node.NodeName;
  attr := Node.AttributeNodes;
  for i := 0 to attr.Count - 1 do
    Result := Result + #32+ attr[i].NodeName + '=''' + attr[i].NodeValue + ''' ';
  Result := Result + '/>'#13#10#13#10':';
  for i := 0 to High(Params) do
    Result := Result + #13#10'  ' + IntToStr(i + 1) + ': ' + Params[i];
end;

function ProcessChild(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList): Boolean;
var
  i: Integer;
  MP: TMacroParams;
begin
  Inc(_NodeLevel);
  for i := 0 to Node.ChildNodes.Count - 1 do
    begin
      ProcessNode(Node.ChildNodes[i], Params, List, MP);
      if MP.Brk then Break;
    end;
  Result := MP.Ret; //   ?
  Dec(_NodeLevel);
end;

procedure ProcessNode(const Node: IXmlNode; var Params: TStringDynArray; const List: TPrayList; var MP: TMacroParams);
var
  Id: string;
  i: Integer;

  {$IFDEF DEBUG}
  procedure _debug;
  var
    id: string;
  begin
    case i of
      0, 17, 18, 19: id := GetAttr(Node, 'cond');
      1: id := GetAttr(Node, 'var') + ' = ' + GetAttr(Node, 'expr');
      2: id := GetAttr(Node, 'var') + ' <- ' + GetAttr(Node, 'index');
      3, 22: id := GetAttr(Node, 'title');
      6: id := GetAttr(Node, 'mode');
      7: id := GetAttr(Node, 'src') + ' -> ' + GetAttr(Node, 'dest');
      10..15, 23, 30, 21: id := GetAttr(Node, 'id');
      28: id := GetAttr(Node, 'var') + GetAttr(Node, 'dest');
      20: id := GetAttr(Node, 'val');
      else id := '';
    end;
    DebugWrite(Node.NodeName + #32 + id, True);
  end;
  {$ENDIF}

begin
  MP.Ret := False;
  MP.Brk := False;
  if Node.NodeType = ntElement then
    try
      i := IndexStr(Node.NodeName, _xstg);
      {$IFDEF DEBUG}
      Id := GetAttr(Node, 'id');
      _debug;
      {$ENDIF}
      if i in [0, 17..19, 25, 26] then
        case i of
          0{if}: begin
            MP.Cond := Condition(Node, Params);
            if MP.Cond then ProcessChild(Node, Params, List);
          end;
          17{while}: while Condition(Node, Params) do ProcessChild(Node, Params, List);
          18{else}: begin
            if not MP.Cond then
              begin
                MP.Cond := Condition(Node, Params);
                if MP.Cond then ProcessChild(Node, Params, List);
              end;
          end;
          19{brif}: MP.Brk := Condition(Node, Params);
        end
      else
        if Condition(Node, Params) then
          case i of
            1: _Calc_Instr(Node, Params);
            2: _Sel_Instr(Node, Params);
            3: _Seq_Instr(Node, Params, List);
            4: _Sync_Instr(Node, Params, List);
            5: _Merge_Instr(Node, Params, List);
            6: _Inter_Instr(Node, Params, List);
            7: _Copy_Instr(Node, Params);
            8: _Text_Instr(Node, Params, List);
            10{add}: AddPray(GetAttr(Node, 'id', Params), Node, Params, List);
            11: _Addall_Instr(GetAttr(Node, 'id', Params), Node, Params, List);
            12{extr}: AddPray(ApplyPV(Id, Params), Node, Params, List, True);
            13{act}: AddAct(GetAttr(Node, 'id', Params), Node, Params, List);
            14{macro}: _Incl_Instr(GetAttr(Node, 'id', Params), Node, Params, List, MacroDic);
            15{proc}: _Incl_Instr(GetAttr(Node, 'id', Params), Node, Params, List, ProcDic);
            23{chain}: _Incl_Instr(GetAttr(Node, 'id', Params), Node, Params, List, ChainDic);
            16: _Rep_Instr(Node, Params, List);
            20{ret}: begin
              if Length(Params) = 0 then SetLength(Params, 1);
              Params[0] := GetAttr(Node, 'val', Params);
              MP.Ret := True;
            end;
            21{rite}: ProcessRite(Id, List);
            22{rqst}: Request(Node);
            27{title}: _Text_Instr(Node, Params, List, True);
            28: _Del_Instr(Node, Params);
            29: _Join_Instr(Node, Params, List);
            30{img}: AddImg(GetAttr(Node, 'id', Params), Node, List);
            {$IFDEF DEBUG}
            24{test}: OutMessage(ApplyPV(Node.Text, Params));
            else raise(Exception.Create(''));
            {$ENDIF}
          end;
    except
      OutMessage(ErrInfo(Node, Params));
    end;
end;

var
  LoadedDics: TStringlist;

procedure LoadDics(const Book: string);
var
  i: Integer;
  p: TPray;
begin
  try
    LoadedDics.Add(Book);
  except
    Exit;
  end;
  try
    XmlCode.LoadFromFile(GetBookPath(Book));
    with XmlCode.DocumentElement.ChildNodes do
      for i := 0 to Count - 1 do
        try
          case IndexStr(Nodes[i].NodeName, _xtg) of
            0: begin
              if GetNumAttr(Nodes[i], 'lang') > 0 then Continue;
              p := TPray.Create;
              p.Mode := 0;
              p.AdjustFromNode(Nodes[i], Book);
              PrayDic.Add(p.Id, p);
            end;
            2: begin
              p := TPray.Create;
              p.Mode := 1;
              p.AdjustFromNode(Nodes[i], Book);
              ActDic.Add(p.Id, p);
            end;
            3: MacroDic.Add(FullId(Book, Nodes[i].GetAttribute('id')), Nodes[i]);
            4: if Book[1] = '.' then ProcDic.Add(Nodes[i].GetAttribute('id'), Nodes[i])
              else ProcDic.Add(FullId(Book, Nodes[i].GetAttribute('id')), Nodes[i]);
            5: RitesDic.Add(Nodes[i].GetAttribute('id'), Nodes[i]);
            6: CompDic.Add(FullId(Book, Nodes[i].GetAttribute('id')), Nodes[i]);
            7: AssemDic.Add(Nodes[i].GetAttribute('id'), Nodes[i]);
            8: TGlory.Load(Nodes[i]);
            9, 10: TPerson.Load(Nodes[i]);
            11: for var j := 0 to Nodes[i].ChildNodes.Count - 1 do
              TMemory.Load(Nodes[i].ChildNodes[j], Nodes[i]);
            12: VarInit(Nodes[i]);
            13: LoadList(Nodes[i]);
            14: LoadPomest(Nodes[i]);
            15: StructDic.Add(FullId(Book, Nodes[i].GetAttribute('id')), Nodes[i]);
            16: ChainDic.Add(FullId(Book, Nodes[i].GetAttribute('id')), Nodes[i]);
            17: begin
              p := TPray.Create;
              p.Mode := 4;
              p.Text := NodeText(Nodes[i]);
              p.Id := Nodes[i].GetAttribute('id');
              p.Alias := Book;
              ImgDic.Add(p.Id, p);
            end;
          end;
        except
          {$IFDEF DEBUG}
          if Nodes[i].NodeName = _xtg[11] then
            OutMessage(' : '#13#10  + _xtg[11] + ' mon=' + GetAttr(Nodes[i], 'mon')  + ' day=' + GetAttr(Nodes[i], 'day'))
          else
   {         OutMessage('   : '#13#10 + Nodes[i].NodeName + ' id=' + GetAttr(Nodes[i], 'id'));}
          {$ENDIF}
        end;
  except
    {$IFDEF DEBUG}
    OutMessage(' : ' + Book);
    {$ENDIF}
  end;
end;

procedure Request(const Node: IXmlNode);
var
  s: string;
begin
  s := GetAttr(Node, 'title');
  if Node.ChildNodes.Count = 0 then
    begin
      {$IFDEF DEBUG}
      OutMessage(' :'#13#10 + s);
      {$ENDIF}
      Exit;
    end;
  if RqVars.ContainsValue(s) then Exit;
  RequestVariants(Node, s);
end;

initialization
  LoadedDics := TStringlist.Create;
  LoadedDics.Sorted := True;
  LoadedDics.Duplicates := dupError;
  GroupDic := TDictionary<string, TGroupRec>.Create;
  MacroDic := TXmlDic.Create;
  StructDic := TXmlDic.Create;
  ProcDic := TXmlDic.Create;
  RitesDic := TXmlDic.Create;
  ChainDic := TXmlDic.Create;
  AssemDic := TXmlDic.Create;
  CompDic := TXmlDic.Create;
finalization
  LoadedDics.Free;
  MacroDic.Free;
  StructDic.Free;
  ProcDic.Free;
  RitesDic.Free;
  AssemDic.Free;
  ChainDic.Free;
  CompDic.Free;
  GroupDic.Free;
end.


