unit EccVars;

interface

uses
  Generics.Collections, SysUtils, Types, XmlIntf;

var
  Vars, RqVars: TDictionary<string, string>;
  SysVars: array[0..43] of string =
{0} ('Display.FirstLetter', 'Display.Perenos', 'Display.LinesMerge',
{3} 'Display.SyncColumns', 'Display.InsertActs', 'Display.Spoilers.Titles',
{6} 'Display.Spoilers', 'Display.Spoilers.Expanded', 'Display.Notbreaks',
{9} 'Lexic.ForceDouble', 'Display.InsertStars', 'Options.SlujChin',
{12}'Options.DiakonsCount', 'Options.TwoLiks', 'Options.IsNastoyatel',
{15}'Options.Monarh.Inside', 'Hram.Path', 'Here.IsPatriarhia',
{18}'Here.IsMonarhia', 'Options.IereysCount', 'Options.ArhiereysCount',
{21}'Display.Snoski', 'Display.Snoski.Kawyka', 'Display.Subscript',
{24}'Display.Subscript.Always', 'Show.MalVecher', 'Show.Vecher',
{27}'Show.Povecher', 'Show.Polunosch', 'Show.Utrenia', 'Show.Chases',
{31}'Show.Chas1', 'Show.Chas36', 'Show.Chas9', 'Show.Mejdochas',
{35}'Show.Izobrazit', 'Show.Liturgia', 'Show.Proskomid', 'Show.Molebens',
{39}'Display.Images', 'Show.Trapeza', 'Interface.Color', 'Interface.LastDate',
{43}'Interface.AutoAssem');

  _LoadStart: array[0..13] of string =
    ('..\defs', '..\lexic', '..\admin', 'ch', 'sl', 'tc', 'tp', 'ps', 'ap', 'ev', 'tb', 'kn', 'ty', 'mi_o');

procedure AdjustSysVars;
procedure VarInit(const Node: IXmlNode);
function CalcExpression(S: string; var Params: TStringDynArray): string;
procedure _Calc_Instr(const Node: IXmlNode; var Params: TStringDynArray);
procedure _Sel_Instr(const Node: IXmlNode; var Params: TStringDynArray);
function Condition(const Node: IXmlNode; var Params: TStringDynArray): Boolean;
procedure StoreVar(const Id, Value: string; const Title: string = '');
procedure DeleteVar(const Id: string);
procedure ClearVarGroup(const Id: string);
function GetVar(const Id: string): string;
function GetIntVar(const Id: string): Integer;
function GetFlag(const Id: string): Boolean;

implementation

uses
  StrUtils, Classes, EccDefs, EccProcs, EccXml, EccCSlav, EccScripts,
  EccAdmin, EccMemos, EccLists, EccPrays;

procedure AdjustSysVars;
begin
  _s_FirstLetter := BoolVar(SysVars[0]);
  _s_Perenos := BoolVar(SysVars[1]);
  _s_LinesMerge := BoolVar(SysVars[2]);
  _s_SyncColumns := BoolVar(SysVars[3]);
  _s_InsertActs := BoolVar(SysVars[4]);
  _s_InsertImgs := BoolVar(SysVars[39]);
  _s_ForceDouble := BoolVar(SysVars[9]);
  _s_Notbreaks := BoolVar(SysVars[8]);
  _o_TwoLiks := BoolVar(SysVars[13]);
  _s_InsertStars := BoolVar(SysVars[10]);
  _s_Spoilers := BoolVar(SysVars[6]);
  _s_SpExpanded := BoolVar(SysVars[7]);
  _s_SpTitles := BoolVar(SysVars[5]);
  DispatchStrArray(Vars['System.TypesWriteSet'], _sTypesWrite);
  DispatchStrArray(Vars['System.StihSet'], _sStih);
  DispatchStrArray(Vars['System.ReplaceStarsSet'], _sReplaceStars);
  DispatchStrArray(Vars['System.NoPretagSet'], _sNoPretag);
  DispatchStrArray(Vars['System.NoLineBreakSet'], _sNoLineBreak);
  DispatchStrArray(Vars['System.NoVoiceAfterStyleSet'], _sNoVoiceAfterStyle);
end;

function ExcludeFromList(List, Value: string): string;
var
  ea: array of string;
  s: string;
  i, j: Integer;
begin
  Result := '';
  if List.IsEmpty then Exit;
  SetLength(ea, Value.CountChar(',') + 1);
  for i := 0 to High(ea) do
    begin
      StrDivide(Value, ea[i], Value, ',');
      ea[i] := Trim(ea[i]);
    end;
  for i := 0 to List.CountChar(',') do
    begin
      StrDivide(List, s, List, ',');
      s := Trim(s);
      if s = '' then Continue;
      for j := 0 to High(ea) do
        if s.Chars[0] in ['0'..'9', '-'] then
          case ea[j].Chars[0] of
            '0'..'9', '-': if s = ea[j] then s := '';
            '*': if SanctsDic[s.ToInteger].Nomin = ea[j].Substring(1).ToInteger then s := '';
            else if GloryDic[ea[j]].IsSanctInList(s.ToInteger) then
              begin
                s := '';
                Break;
              end;
          end
        else
          case ea[j].Chars[0] of
            '0'..'9', '-': if GloryDic[s].IsSanctInList(ea[j].ToInteger) and (GloryDic[s].Stared = 1) then s := '';
            '*': if GloryDic[s].Grade = ea[j].Substring(1).ToInteger then s := '';
            else if s = ea[j] then
              begin
                s := '';
                Break;
              end;
          end;
      if s <> '' then Result := Result + s + ', ';
    end;
  Result := Result.Remove(Result.Length - 2);
  SetLength(ea, 0);
end;

procedure _Calc_Instr(const Node: IXmlNode; var Params: TStringDynArray);
var
  s, w: string;
  n: Integer;
begin
  s := GetAttr(Node, 'var', Params);
  try
    w := GetAttr(Node, 'expr', Params);
    if s.Chars[0] = '_' then
      begin
        n := s.Substring(1).ToInteger;
        if Length(Params) < n then SetLength(Params, n);
        Params[n - 1] := w;
      end
    else Vars.AddOrSetValue(s, w);
    {$IFDEF DEBUG}
    DebugWrite('= ' + w, True);
    {$ENDIF}
  except
    {$IFDEF DEBUG}
    OutMessage('   : '#13#10 + s + ' = ' + GetAttr(Node, 'expr'));
    {$ENDIF}
  end;
end;

function InternalFunc(S, n: string): string;
var
  p: string;
begin
  StrDivide(n, n, p, ':');
  case IndexStr(n, ['lexic', 'pripev', 'name', 'role', 'gend']) of
    0: Result := ApplyLexic(S, p.ToInteger);
    1: begin
      var pt: TPhraseTemplate := PripevsList[GloryDic[p].TypeNum];
      Result := ApplyLexic(pt.Text[LexCount(GloryDic[p].FSancts.Count)].Replace('[*]', GloryDic[p].Call), pt.Pad);
    end;
    2: Result := CutLexic(S, 'N');
    3: Result := CutLexic(S, 'R');
    4: Result := NamesList[CutLexic(S, 'N').ToInteger].Gen.ToString;
    {$IFDEF DEBUG}
    else OutMessage('  : ' + n + #32);
    {$ENDIF}
  end;
end;

procedure VarInit(const Node: IXmlNode);
var
  id, value, pp: string;
begin
  id := GetAttr(Node, 'id');
  if Node.Text <> '' then value := Node.Text
  else value := GetAttr(Node, 'val');
  Vars.AddOrSetValue(id, value);
  pp := GetAttr(Node, 'title');
  if pp <> '' then RqVars.AddOrSetValue(id, pp);
end;

procedure _Sel_Instr(const Node: IXmlNode; var Params: TStringDynArray);
var
  id, value: string;
  sl: TStringList;
  n: Integer;
begin
  try
    sl := TStringList.Create;
    try
      n := GetAttr(Node, 'index', Params).ToInteger;
    except
      n := 0;
    end;
    sl.Text := ApplyPV(ApplyFlags(Node.Text), Params);
    try
      value := Trim(sl[n]);
    except
      value := '';
    end;
    id := GetAttr(Node, 'var', Params);
    if id.Chars[0] = '_' then
      begin
        n := id.Substring(1).ToInteger;
        if Length(Params) < n then SetLength(Params, n);
        Params[n - 1] := value;
      end
    else Vars.AddOrSetValue(id, value);
    {$IFDEF DEBUG}
    DebugWrite('= ' + value, True);
    {$ENDIF}
  finally
    sl.Free;
  end;
end;

function CutFirst(var S: string): string;
var
  i, c: Integer;
begin
  if S.IsEmpty then Exit('');
  if S.Chars[0] <> '(' then StrDivide(S, Result, S, #32)
  else
    begin
      c := 0;
      for i := 0 to S.Length - 1 do
        case S.Chars[i] of
          ')': begin
            Dec(c);
            if c = 0 then Break;
          end;
          '(': Inc(c);
        end;
      if i = (S.Length - 1) then
        begin
          S := S.Substring(1, S.Length - 2);
          Result := CutFirst(S);
        end
      else
        begin
          Result := S.Substring(0, i + 1);
          S := S.Substring(i + 2);
        end;
    end;
end;

function CalcExpression(S: string; var Params: TStringDynArray): string;
var
  a1, o, a2, w, ri: string;
  i, r: Integer;
begin
  Result := '0';
  if S.IsEmpty then Exit(a1);
  a1 := CutFirst(S);
  a2 := S;
  if (a1 = '-') or (a1 = '+') or (a1.Chars[0] = '$') then
    begin
      o := a1;
      a1 := '&';
    end
  else
    begin
      StrDivide(a2, o, a2, #32);
      a1 := ApplyPV(a1, Params);
    end;
  a2 := ApplyPV(a2, Params);
  if o.Chars[0] = '$' then
    begin
      if a2.Chars[a2.Length - 2] = ':' then
        begin
          StrDivide(a2, a2, ri, ':', True);
          r := ri.ToInteger;
        end;
      StrDivide(o.Substring(1), o, ri, '.');
    end;
  i := IndexStr(o, ['-', '+', '=', '#', '/', '\', '*', ':', '.',
   '%', '!', '|', 'pray', 'serv', 'glory', 'list', 'func', 'group']);
  {$IFDEF DEBUG}
  if i < 0 then
    begin
      OutMessage('   : ' + s);
      Exit;
    end;
  {$ENDIF}
  try
    if (i in [0..1]) then
      begin
        if a2.IsEmpty then Exit((not Boolean(i)).ToString);
        case a2.Chars[0] of
          '^': Exit(Ord(Ord(GetServElm(a2).Count > 0) = i).ToString);
          '@': begin
            StrDivide(a2, a2, w, ':');
            if w.IsEmpty then r := 0 else r := w.ToInteger;
            Exit(Ord(Ord(Assigned(FindGroupInExternService(a2.Substring(1), r))) = i).ToString);
          end;
          '_': begin
            r := a2.Substring(1).ToInteger;
            Exit(Ord((Length(Params) >= r) and (not Params[r - 1].IsEmpty)).ToString);
          end;
        end;
      end;
    case i of
      0: if a1 = '&' then Result := Ord(a2 = '0').ToString
      else Result := (a1.ToInteger - a2.ToInteger).ToString;
      1: if a1 = '&' then Result := (a2 <> '0').ToString
      else Result := (a1.ToInteger + a2.ToInteger).ToString;
      2: Result := Ord(a1 = a2).ToString;
      3: Result := Ord(a1 <> a2).ToString;
      4: Result := Ord(a1.ToInteger < a2.ToInteger).ToString;
      5: Result := Ord(a1.ToInteger > a2.ToInteger).ToString;
      6: Result := (a1.ToInteger * a2.ToInteger).ToString;
      7: Result := (a1.ToInteger div a2.ToInteger).ToString;
      8: Result := (a1.ToInteger mod a2.ToInteger).ToString;
      9: Result := ExcludeFromList(a1, a2);
      10: Result := Ord(a1.ToBoolean and a2.ToBoolean).ToString;
      11: Result := Ord(a1.ToBoolean or a2.ToBoolean).ToString;
      12: Result := GetPrayParam(a2, ri, r);
      13: Result := GetServParam(a2, ri);
      14: Result := GetGloryParam(a2, ri);
      15: Result := TStringDynArray(FindStringArray(ri)^)[a2.ToInteger];
      16: Result := InternalFunc(a2, ri);
      17: Result := GetPrayParam(a2 + '.-1', 'index', r);
    end;
  except
    {$IFDEF DEBUG}
    OutMessage('  : '  + s);
    {$ENDIF}
  end;
end;

function Condition(const Node: IXmlNode; var Params: TStringDynArray): Boolean;
var
  s: string;
begin
  s := GetAttr(Node, 'cond');
  if s.IsEmpty then Exit(True);
  s := CalcExpression('(' + s + ')', Params);
  Result := (s <> '0') and not s.IsEmpty;
end;

procedure StoreVar(const Id, Value: string; const Title: string = '');
var
  node: IXmlNode;
begin
  XmlCode.LoadFromFile(GetBookPath(_LoadStart[0]));
  node := FindOrCreateNode(XmlCode.DocumentElement, _xtg[12], Id);
  AssignAttr(node, 'val', Value);
  AssignAttr(node, 'title', Title);
  XmlCode.SaveToFile(GetBookPath(_LoadStart[0]));
end;

procedure DeleteVar(const Id: string);
begin
  XmlCode.LoadFromFile(GetBookPath(_LoadStart[0]));
  DeleteNodeWithId(XmlCode.DocumentElement, _xtg[12], Id);
  XmlCode.SaveToFile(GetBookPath(_LoadStart[0]));
end;

function GetVar(const Id: string): string;
begin
  try
    Result := Vars[Id];
  except
    Result := '';
  end;
end;

function GetIntVar(const Id: string): Integer;
begin
  try
    Result := Vars[Id].ToInteger;
  except
    Result := 0;
  end;
end;

function GetFlag(const Id: string): Boolean;
begin
  Result := GetIntVar(Id) > 0;
end;

procedure ClearVarGroup(const Id: string);
var
  vr: TPair<string, string>;
begin
  for vr in Vars do
    if vr.Key.Substring(0, Length(Id) + 1) = (Id + '.') then Vars.Remove(vr.Key);
end;

initialization
  Vars := TDictionary<string, string>.Create;
  RqVars := TDictionary<string, string>.Create;
finalization
  Vars.Free;
  RqVars.Free;
end.
