unit EccXml;

interface

uses
  Types, XmlIntf;

function GetAttr(const Node: IXmlNode; const AttrName: string): string; overload;
function GetNumAttr(const Node: IXmlNode; const AttrName: string): Integer; overload;
function GetAttr(const Node: IXmlNode; const AttrName: string; const Params: TStringDynArray): string; overload;
function GetNumAttr(const Node: IXmlNode; const AttrName: string; const Params: TStringDynArray): Integer; overload;
function NodeText(const Node: IXmlNode): string;
procedure AssignAttr(const N: IXmlNode; const s: string; i: Integer); overload;
procedure AssignAttr(const N: IXmlNode; const s, i: string); overload;
function FindNodeWithId(const Node: IXmlNode; const Name, Id: string): IXmlNode;
function DeleteNodeWithId(const Node: IXmlNode; const Name, Id: string): Boolean;
function FindOrCreateNode(const Node: IXmlNode; const Name, Id: string): IXmlNode;
function FindNodeByPath(P: string): IXmlNode;

implementation

uses
  SysUtils, Variants, EccScripts, EccProcs;

function GetAttr(const Node: IXmlNode; const AttrName: string): string; overload;
begin
  if Node.HasAttribute(AttrName) then
    Result := Node.GetAttribute(AttrName)
  else Result := '';
end;

function GetNumAttr(const Node: IXmlNode; const AttrName: string): Integer; overload;
begin
  if Node.HasAttribute(AttrName) then
    Result := Node.GetAttribute(AttrName)
  else Result := 0;
end;

function GetAttr(const Node: IXmlNode; const AttrName: string; const Params: TStringDynArray): string; overload;
begin
  if Node.HasAttribute(AttrName) then
    Result := ApplyPV(ApplyFlags(Node.GetAttribute(AttrName)), Params)
  else Result := '';
end;

function GetNumAttr(const Node: IXmlNode; const AttrName: string; const Params: TStringDynArray): Integer; overload;
begin
  if Node.HasAttribute(AttrName) then
    Result := ToInt(GetAttr(Node, AttrName, Params))
  else Result := 0;
end;

function NodeText(const Node: IXmlNode): string;
begin
  try
    Result := Trim(Node.NodeValue);
  except
    Result := '';
  end;
end;

procedure AssignAttr(const N: IXmlNode; const s: string; i: Integer); overload;
begin
  if i > 0 then n.Attributes[s] := i
  else n.Attributes[s] := Null;
end;

procedure AssignAttr(const N: IXmlNode; const s, i: string); overload;
begin
  if i <> '' then n.Attributes[s] := i
  else n.Attributes[s] := Null;
end;

function FindNodeWithId(const Node: IXmlNode; const Name, Id: string): IXmlNode;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Node.ChildNodes.Count - 1 do
    if ((Node.ChildNodes[i].NodeName = Name) or (Name = '')) and (GetAttr(Node.ChildNodes[i], 'id') = Id) then
      begin
        Result := Node.ChildNodes[i];
        Exit;
      end;
end;

function DeleteNodeWithId(const Node: IXmlNode; const Name, Id: string): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Node.ChildNodes.Count - 1 do
    if ((Node.ChildNodes[i].NodeName = Name) or (Name = '')) and (GetAttr(Node.ChildNodes[i], 'id') = Id) then
      begin
        Result := True;
        Node.ChildNodes.Delete(i);
        Exit;
      end;
end;

function FindOrCreateNode(const Node: IXmlNode; const Name, Id: string): IXmlNode;
begin
  Result := FindNodeWithId(Node, Name, id);
  if Result = nil then
    begin
      Result := Node.AddChild(Name);
      Result.Attributes['id'] := id;
    end;
end;

function FindNodeByPath(P: string): IXmlNode;
var
  s: string;
begin
  Result := XmlCode.DocumentElement;
  while Assigned(Result) and not P.IsEmpty do
    begin
      StrDivide(P, s, P, '.');
      if Result = XmlCode.DocumentElement then
        Result := FindNodeWithId(Result, 'pomest', s)
      else
        Result := FindNodeWithId(Result, '', s);
    end;
end;

end.
